package com.aware.plugin.beacons;

import android.app.Fragment;
import android.app.FragmentManager;
import android.content.res.Configuration;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.v7.app.ActionBarActivity;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.widget.Toolbar;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.util.Log;
import android.view.*;
import android.widget.*;
import com.aware.Aware;
import com.aware.Aware_Preferences;
import com.aware.plugin.beacons.view.*;

import java.util.ArrayList;
import java.util.List;

import static com.aware.plugin.beacons.Constants.CALIBRATION_FRAGMENT_EXTRA;

public class Settings extends ActionBarActivity {

    private static final String BUNDLE_FRAGMENT_COUNTER = "FRAGMENT_COUNTER";

    private boolean DEBUG;

    private DrawerLayout mDrawerLayout;
    private ListView mDrawerList;
    private ActionBarDrawerToggle mDrawerToggle;

    private CharSequence mDrawerTitle;
    private CharSequence mTitle;
    private String[] mDrawerItemLabels;

    private int red = 8;
    private int green = 120;
    private int blue = 165;

    private int redRange = 20; //20
    private int greenRange = 30; //30
    private int blueRange = 40; //40

    private int fragmentCounter;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        fragmentCounter = savedInstanceState == null ? 0 : savedInstanceState.getInt(BUNDLE_FRAGMENT_COUNTER);

        if(getIntent().getBooleanExtra(CALIBRATION_FRAGMENT_EXTRA, false)){
            fragmentCounter = 1;
        }

        setContentView(R.layout.main_activity_layout);

        PreferenceManager.setDefaultValues(this, R.xml.settings, false);

        mTitle = mDrawerTitle = getTitle();
        mDrawerItemLabels = new String[]{"Beacons",
                "Calibration",
                "Map",
                "Settings"};
        mDrawerLayout = (DrawerLayout) findViewById(R.id.drawer_layout);
        mDrawerList = (ListView) findViewById(R.id.left_drawer);

        mDrawerLayout.setDrawerShadow(R.drawable.drawer_shadow, GravityCompat.START);

        List<DrawerItemListAdapter.LabelIconPair> items = prepareLabelIconList(fragmentCounter);

        mDrawerList.setAdapter(new DrawerItemListAdapter(this,
                R.layout.drawer_list_item,
                android.R.id.text1,
                items));

        mDrawerList.setOnItemClickListener(new DrawerItemClickListener());

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        mDrawerToggle = new ActionBarDrawerToggle(
                this,
                mDrawerLayout,
                toolbar,
                R.string.drawer_open,
                R.string.drawer_close
        ) {
            public void onDrawerClosed(View view) {
                super.onDrawerClosed(view);
                getSupportActionBar().setTitle(mTitle);
                invalidateOptionsMenu();
                getSupportActionBar().setBackgroundDrawable(new ColorDrawable(
                        Color.rgb(red, green, blue)));
            }

            public void onDrawerOpened(View drawerView) {
                super.onDrawerOpened(drawerView);
                getSupportActionBar().setTitle(mDrawerTitle);
                invalidateOptionsMenu();
                getSupportActionBar().setBackgroundDrawable(new ColorDrawable(
                        Color.rgb(red + redRange, green - greenRange, blue - blueRange)));
            }

            @Override
            public void onDrawerSlide(View view, float v) {
                super.onDrawerSlide(view, v);
                getSupportActionBar().setBackgroundDrawable(new ColorDrawable(
                        Color.rgb(red + (int)(v * redRange), green - (int)(v * greenRange), blue - (int)(v * blueRange))));
            }
        };
        mDrawerLayout.setDrawerListener(mDrawerToggle);

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setHomeButtonEnabled(true);

        getSupportActionBar().setBackgroundDrawable(new ColorDrawable(Color.rgb(red, green, blue)));

        DEBUG = Aware.getSetting(this.getContentResolver(), Aware_Preferences.DEBUG_FLAG).equals("true");

        selectItem(fragmentCounter);

        if(DEBUG){
            Log.d(Constants.BEACONS_PLUGIN_TAG, "Settings created");
        }
    }

    @Override
    protected void onDestroy(){
        super.onDestroy();
    }

    private List<DrawerItemListAdapter.LabelIconPair> prepareLabelIconList(int lightPosition){
        List<DrawerItemListAdapter.LabelIconPair> items = new ArrayList<>();
        for(int i = 0; i < mDrawerItemLabels.length; i++){
            if(i == lightPosition){
                items.add(new DrawerItemListAdapter.LabelIconPair(mDrawerItemLabels[i], "light"));
            }else{
                items.add(new DrawerItemListAdapter.LabelIconPair(mDrawerItemLabels[i], "dark"));
            }
        }
        return items;
    }

    private class DrawerItemClickListener implements ListView.OnItemClickListener {
        @Override
        public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
            List<DrawerItemListAdapter.LabelIconPair> items = prepareLabelIconList(position);
            DrawerItemListAdapter adapter = (DrawerItemListAdapter) parent.getAdapter();
            adapter.setItems(items);
            adapter.notifyDataSetChanged();
            selectItem(position);
        }
    }

    private void selectItem(int position) {
        fragmentCounter = position;
        Fragment fragment;
        switch (position){
            case 0:
                fragment = new BeaconListFragment();
                break;
            case 1:
                fragment = new CalibrationReadingsListFragment();
                break;
            case 2:
                fragment = new MapFragment();
                break;
            case 3:
                fragment = new BeaconsPreferenceFragment();
                break;
            default:
                return;
        }
        FragmentManager fragmentManager = getFragmentManager();
        fragmentManager.beginTransaction().replace(R.id.content_frame, fragment).commit();

        mDrawerList.setItemChecked(position, true);
        setTitle(mDrawerItemLabels[position]);
        mDrawerLayout.closeDrawer(mDrawerList);
    }

    @Override
    public void setTitle(CharSequence title) {
        mTitle = title;
        getSupportActionBar().setTitle(mTitle);
    }

    @Override
    protected void onPostCreate(Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        mDrawerToggle.syncState();
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        mDrawerToggle.onConfigurationChanged(newConfig);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (mDrawerToggle.onOptionsItemSelected(item)) {
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putInt(BUNDLE_FRAGMENT_COUNTER, fragmentCounter);
    }
}
