package com.aware.plugin.beacons;

import com.aware.plugin.beacons.model.AbstractBeacon;
import com.aware.plugin.beacons.model.CalibrationReading;
import com.aware.plugin.beacons.semweb.JSONBeacon;

import java.util.*;

public class Singleton {
    private static Singleton ourInstance = new Singleton();

    private Set<AbstractBeacon> foundDevices = new HashSet<>();
    private Set<AbstractBeacon> selectedDevices = new HashSet<>();

    private Map<Integer, List<CalibrationReading>> calibrationData = new HashMap<>();

    private List<JSONBeacon> similarBeacons = new ArrayList<>();

    private Singleton() {
    }

    public static Singleton getInstance() {
        return ourInstance;
    }

    public Set<AbstractBeacon> getFoundDevices() {
        return foundDevices;
    }

    public void addFoundDevice(AbstractBeacon device){
        foundDevices.add(device);
    }

    public void removeFoundDevice(AbstractBeacon device){
        foundDevices.remove(device);
    }

    public AbstractBeacon getExistingBeacon(AbstractBeacon beacon){
        for(AbstractBeacon b : foundDevices){
            if(((Object)beacon).equals(b)){
                return b;
            }
        }
        return null;
    }

    public Set<AbstractBeacon> getSelectedDevices() {
        return selectedDevices;
    }

    public void addSelectedDevice(AbstractBeacon device){
        selectedDevices.add(device);
    }

    public void removeSelectedDevice(AbstractBeacon device){
        selectedDevices.remove(device);
    }

    public Map<Integer, List<CalibrationReading>> getCalibrationData() {
        return calibrationData;
    }

    public void setCalibrationData(Map<Integer, List<CalibrationReading>> calibrationData) {
        this.calibrationData = calibrationData;
    }

    public void addCalibrationReading(Integer key, CalibrationReading reading){
        List<CalibrationReading> list = calibrationData.get(key);
        if(list == null){
            list = new ArrayList<>();
            calibrationData.put(key, list);
        }
        list.add(reading);
    }

    public AbstractBeacon getBeaconByMacAddress(String macAddress){
        for(AbstractBeacon beacon : foundDevices){
            if(beacon.getMacAddress().equals(macAddress)){
                return beacon;
            }
        }
        return null;
    }

    public int getCalibrationDataCount(){
        int count = 0;
        for (List<CalibrationReading> coll : calibrationData.values()){
            count += coll.size();
        }
        return count;
    }

    public AbstractBeacon getClosestBeacon() {
        double maxRssi = -100;
        AbstractBeacon closestBeacon = null;
        for(AbstractBeacon beacon : foundDevices){
            if(beacon.getRssi() > maxRssi){
                maxRssi = beacon.getRssi();
                closestBeacon = beacon;
            }
        }
        return closestBeacon;
    }

    public List<JSONBeacon> getSimilarBeacons() {
        return similarBeacons;
    }

    public void setSimilarBeacons(List<JSONBeacon> similarBeacons) {
        this.similarBeacons = similarBeacons;
    }
}
