package com.aware.plugin.beacons.db;

import android.content.BroadcastReceiver;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.util.Log;
import com.aware.Aware;
import com.aware.Aware_Preferences;
import com.aware.plugin.beacons.Constants;
import com.aware.plugin.beacons.Singleton;
import com.aware.plugin.beacons.model.AbstractBeacon;

import static com.aware.plugin.beacons.Constants.BEACON_EXTRA;
import static com.aware.plugin.beacons.Constants.ACTION_AWARE_BEACONS;

import java.util.Arrays;

public class BeaconScanBroadcastReceiver extends BroadcastReceiver {

    @Override
    public void onReceive(Context context, Intent intent) {

        AbstractBeacon beacon = intent.getParcelableExtra(BEACON_EXTRA);

        if(beacon != null && Singleton.getInstance().getSelectedDevices().contains(beacon)){

            boolean DEBUG = Aware.getSetting(context.getContentResolver(), Aware_Preferences.DEBUG_FLAG).equals("true");

            ContentValues rowData = new ContentValues();

            rowData.put(Beacons_Provider.Beacon.TIMESTAMP,
                    System.currentTimeMillis());
            rowData.put(Beacons_Provider.Beacon.DEVICE_ID,
                    Aware.getSetting(context.getContentResolver(), Aware_Preferences.DEVICE_ID));
            rowData.put(Beacons_Provider.Beacon.DEVICE_ADDRESS,
                    beacon.getMacAddress());
            rowData.put(Beacons_Provider.Beacon.RSSI,
                    beacon.getRssi());
            rowData.put(Beacons_Provider.Beacon.NUMBER_OF_BYTES_FOLLOWING_FIRST_AD_STRUCTURE,
                    beacon.getScanResult().getNumberOfBytesFollowingFirstADStructure());
            rowData.put(Beacons_Provider.Beacon.AD_TYPE,
                    beacon.getScanResult().getADType());
            rowData.put(Beacons_Provider.Beacon.FLAG,
                    beacon.getScanResult().getFlag());
            rowData.put(Beacons_Provider.Beacon.LE_LIMITED_DISCOVERABLE_MODE,
                    beacon.getScanResult().isLELimitedDiscoverableMode());
            rowData.put(Beacons_Provider.Beacon.LE_GENERAL_DISCOVERABLE_MODE,
                    beacon.getScanResult().isLEGeneralDiscoverableMode());
            rowData.put(Beacons_Provider.Beacon.BR_EDR_NOT_SUPPORTED,
                    beacon.getScanResult().isBREDRNotSupported());
            rowData.put(Beacons_Provider.Beacon.SIMULTANEOUS_LE_AND_BR_EDR_TO_SAME_DEVICE_CAPABLE_CONTROLLER,
                    beacon.getScanResult().isSimultaneousLEandBREDRtoSameDeviceCapableController());
            rowData.put(Beacons_Provider.Beacon.SIMULTANEOUS_LE_AND_BR_EDR_TO_SAME_DEVICE_CAPABLE_HOST,
                    beacon.getScanResult().isSimultaneousLEandBREDRtoSameDeviceCapableHost());
            rowData.put(Beacons_Provider.Beacon.NUMBER_OF_BYTES_FOLLOWING_SECOND_AD_STRUCTURE,
                    beacon.getScanResult().getNumberOfBytesFollowingSecondADStructure());
            rowData.put(Beacons_Provider.Beacon.MANUFACTURER_SPECIFIC_DATA_AD_TYPE,
                    beacon.getScanResult().getManufacturerSpecificDataADType());
            rowData.put(Beacons_Provider.Beacon.COMPANY_IDENTIFIER_CODE,
                    Arrays.toString(beacon.getScanResult().getCompanyIdentifierCode()));
            rowData.put(Beacons_Provider.Beacon.BYTE_0_OF_IBEACON_ADVERTISEMENT_INDICATOR,
                    beacon.getScanResult().getByte0ofiBeaconAdvertisementIndicator());
            rowData.put(Beacons_Provider.Beacon.BYTE_1_OF_IBEACON_ADVERTISEMENT_INDICATOR,
                    beacon.getScanResult().getByte1ofiBeaconAdvertisementIndicator());
            rowData.put(Beacons_Provider.Beacon.PROXIMITY_UUID,
                    Arrays.toString(beacon.getScanResult().getProximityUuid()));
//            rowData.put(Beacons_Provider.Beacon.MAJOR,
//                    Arrays.toString(result.getMajor()));
//            rowData.put(Beacons_Provider.Beacon.MINOR,
//                    Arrays.toString(result.getMinor()));
            rowData.put(Beacons_Provider.Beacon.MAJOR,
                    beacon.getMajor());
            rowData.put(Beacons_Provider.Beacon.MINOR,
                    beacon.getMinor());
            rowData.put(Beacons_Provider.Beacon.TWOS_COMPLEMENT_OF_CALIBRATED_TX_POWER,
                    beacon.getScanResult().getTwosComplementOfCalibratedTxPower());
            rowData.put(Beacons_Provider.Beacon.CHECKSUM,
                    Arrays.toString(beacon.getScanResult().getChecksum()));
            rowData.put(Beacons_Provider.Beacon.FORMATTED_PROXIMITY_UUID,
                    beacon.getScanResult().getFormattedProximityUuid());
            rowData.put(Beacons_Provider.Beacon.FORMATTED_MAJOR,
                    beacon.getScanResult().getFormattedMajor());
            rowData.put(Beacons_Provider.Beacon.FORMATTED_MINOR,
                    beacon.getScanResult().getFormattedMinor());

            context.getContentResolver().insert(Beacons_Provider.Beacon.CONTENT_URI, rowData);

            if(DEBUG) {
                Log.d(Constants.BEACONS_PLUGIN_TAG, rowData.toString());
                Log.d(Aware.TAG, rowData.toString());
            }

            Intent share = new Intent(ACTION_AWARE_BEACONS);
            context.sendBroadcast(share);
        }
    }
}
