package com.aware.plugin.beacons.db;

import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteQueryBuilder;
import android.net.Uri;
import android.os.Environment;
import android.provider.BaseColumns;
import android.util.Log;
import com.aware.Aware;
import com.aware.utils.DatabaseHelper;

import java.util.HashMap;

public class Beacons_Provider extends ContentProvider {

    public static final String AUTHORITY = "com.aware.provider.plugin.beacons";

    private static final int DATABASE_VERSION = 2;

    private static final int BEACONS = 1;
    private static final int BEACONS_ID = 2;

    private static final int EXPERIMENTS = 3;
    private static final int EXPERIMENTS_ID = 4;

    private static final int CALIBRATION_READINGS_BASE = 5;
    private static final int CALIBRATION_READINGS_BASE_ID = 6;

    private static final int CALIBRATION_READINGS = 7;
    private static final int CALIBRATION_READINGS_ID = 8;

    private static UriMatcher uriMatcher = null;
    private static DatabaseHelper databaseHelper = null;
    private static SQLiteDatabase database = null;

    private static HashMap<String, String> beaconMap = null;
    private static HashMap<String, String> experimentMap = null;
    private static HashMap<String, String> calibrationReadingsBaseMap = null;
    private static HashMap<String, String> calibrationReadingsMap = null;

    public static final class Beacon implements BaseColumns {
        private Beacon() {}

        public static final Uri CONTENT_URI = Uri.parse("content://"+AUTHORITY+"/plugin_beacons_beacons"); //this needs to match the table name
        public static final String CONTENT_TYPE = "vnd.android.cursor.dir/vnd.aware.plugin.beacons.beacon";
        public static final String CONTENT_ITEM_TYPE = "vnd.android.cursor.item/vnd.aware.plugin.beacons.beacon";

        public static final String _ID = "_id";
        public static final String TIMESTAMP = "timestamp";
        public static final String DEVICE_ID = "device_id";

        public static final String DEVICE_ADDRESS = "device_address";
        public static final String RSSI = "rssi";
        public static final String NUMBER_OF_BYTES_FOLLOWING_FIRST_AD_STRUCTURE = "number_of_bytes_following_first_ad_structure";
        public static final String AD_TYPE = "ad_type";
        public static final String FLAG = "flag";
        public static final String LE_LIMITED_DISCOVERABLE_MODE = "le_limited_discoverable_mode";
        public static final String LE_GENERAL_DISCOVERABLE_MODE = "le_general_discoverable_mode";
        public static final String BR_EDR_NOT_SUPPORTED = "br_edr_not_supported";
        public static final String SIMULTANEOUS_LE_AND_BR_EDR_TO_SAME_DEVICE_CAPABLE_CONTROLLER = "simultaneous_le_and_br_edr_to_same_device_capable_controller";
        public static final String SIMULTANEOUS_LE_AND_BR_EDR_TO_SAME_DEVICE_CAPABLE_HOST = "simultaneous_le_and_br_edr_to_same_device_capable_host";
        public static final String NUMBER_OF_BYTES_FOLLOWING_SECOND_AD_STRUCTURE = "number_of_bytes_following_second_ad_structure";
        public static final String MANUFACTURER_SPECIFIC_DATA_AD_TYPE = "manufacturer_specific_data_ad_type";
        public static final String COMPANY_IDENTIFIER_CODE = "company_identifier_code";
        public static final String BYTE_0_OF_IBEACON_ADVERTISEMENT_INDICATOR = "byte_0_of_ibeacon_advertisement_indicator";
        public static final String BYTE_1_OF_IBEACON_ADVERTISEMENT_INDICATOR = "byte_1_of_ibeacon_advertisement_indicator";
        public static final String PROXIMITY_UUID  = "proximity_uuid";
        public static final String MAJOR = "major";
        public static final String MINOR = "minor";
        public static final String TWOS_COMPLEMENT_OF_CALIBRATED_TX_POWER = "twos_complement_of_calibrated_tx_power";
        public static final String CHECKSUM = "checksum";

        public static final String FORMATTED_PROXIMITY_UUID = "formatted_proximity_uuid";
        public static final String FORMATTED_MAJOR = "formatted_major";
        public static final String FORMATTED_MINOR = "formatted_minor";
    }

    public static final class Experiment implements BaseColumns {
        private Experiment() {}

        public static final Uri CONTENT_URI = Uri.parse("content://"+AUTHORITY+"/plugin_beacons_experiments"); //this needs to match the table name
        public static final String CONTENT_TYPE = "vnd.android.cursor.dir/vnd.aware.plugin.beacons.experiment";
        public static final String CONTENT_ITEM_TYPE = "vnd.android.cursor.item/vnd.aware.plugin.beacons.experiment";

        public static final String _ID = "_id";
        public static final String TIMESTAMP = "timestamp";
        public static final String DEVICE_ID = "device_id";

        public static final String LABEL = "label";
        public static final String START = "double_start";
        public static final String STOP = "double_stop";
    }

    public static final class Calibration_Readings_Base implements BaseColumns {
        private Calibration_Readings_Base(){}

        public static final Uri CONTENT_URI = Uri.parse("content://"+AUTHORITY+"/plugin_beacons_calibrationreadings_base");
        public static final String CONTENT_TYPE = "vnd.android.cursor.item/vnd.aware.plugin.beacons.calibrationreadings.base";
        public static final String CONTENT_ITEM_TYPE = "vnd.android.cursor.item/vnd.aware.plugin.beacons.calibrationreadings.base";

        public static final String _ID = "_id";
        public static final String TIMESTAMP = "timestamp";
        public static final String DEVICE_ID = "device_id";
        public static final String NUMBER_OF_READINGS = "number_of_readings";
    }

    public static final class Calibration_Readings implements BaseColumns{
        private Calibration_Readings(){}

        public static final Uri CONTENT_URI = Uri.parse("content://"+AUTHORITY+"/plugin_beacons_calibrationreadings");
        public static final String CONTENT_TYPE = "vnd.android.cursor.item/vnd.aware.plugin.beacons.calibrationreadings";
        public static final String CONTENT_ITEM_TYPE = "vnd.android.cursor.item/vnd.aware.plugin.beacons.calibrationreadings";

        public static final String _ID = "_id";
        public static final String BASE_ID = "base_id";
        public static final String TIMESTAMP = "timestamp";
        public static final String RSSI = "rssi";
        public static final String DISTANCE = "distance";
    }

    public static String DATABASE_NAME = Environment.getExternalStorageDirectory() + "/AWARE/plugin_beacons.db";

    public static final String[] DATABASE_TABLES = {
            "plugin_beacons_beacons",
            "plugin_beacons_experiments",
            "plugin_beacons_calibrationreadings_base",
            "plugin_beacons_calibrationreadings"
    };

    public static final String[] TABLES_FIELDS = {
            Beacon._ID + " integer primary key autoincrement," +
                    Beacon.TIMESTAMP + " real default 0," +
                    Beacon.DEVICE_ID + " text default ''," +
                    Beacon.DEVICE_ADDRESS + " text default ''," +
                    Beacon.RSSI + " integer default 0," +
                    Beacon.NUMBER_OF_BYTES_FOLLOWING_FIRST_AD_STRUCTURE + " integer default 0," +
                    Beacon.AD_TYPE + " integer default 0," +
                    Beacon.FLAG + " integer default 0," +
                    Beacon.LE_LIMITED_DISCOVERABLE_MODE + " integer default 0," +
                    Beacon.LE_GENERAL_DISCOVERABLE_MODE + " integer default 0," +
                    Beacon.BR_EDR_NOT_SUPPORTED + " integer default 0," +
                    Beacon.SIMULTANEOUS_LE_AND_BR_EDR_TO_SAME_DEVICE_CAPABLE_CONTROLLER + " integer default 0," +
                    Beacon.SIMULTANEOUS_LE_AND_BR_EDR_TO_SAME_DEVICE_CAPABLE_HOST + " integer default 0," +
                    Beacon.NUMBER_OF_BYTES_FOLLOWING_SECOND_AD_STRUCTURE + " integer default 0," +
                    Beacon.MANUFACTURER_SPECIFIC_DATA_AD_TYPE + " integer default 0," +
                    Beacon.COMPANY_IDENTIFIER_CODE + " text default ''," +
                    Beacon.BYTE_0_OF_IBEACON_ADVERTISEMENT_INDICATOR + " integer default 0," +
                    Beacon.BYTE_1_OF_IBEACON_ADVERTISEMENT_INDICATOR + " integer default 0," +
                    Beacon.PROXIMITY_UUID + " text default ''," +
                    Beacon.MAJOR + " text default ''," +
                    Beacon.MINOR + " text default ''," +
                    Beacon.TWOS_COMPLEMENT_OF_CALIBRATED_TX_POWER + " integer default 0," +
                    Beacon.CHECKSUM + " text default ''," +
                    Beacon.FORMATTED_PROXIMITY_UUID + " text default ''," +
                    Beacon.FORMATTED_MAJOR + " text default ''," +
                    Beacon.FORMATTED_MINOR + " text default ''," +
                    "UNIQUE ("+Beacon.TIMESTAMP+","+Beacon.DEVICE_ID+")",
            Experiment._ID + " integer primary key autoincrement," +
                    Experiment.TIMESTAMP + " real default 0," +
                    Experiment.DEVICE_ID + " text default ''," +
                    Experiment.LABEL + " text default ''," +
                    Experiment.START + " real default 0," +
                    Experiment.STOP + " real default 0," +
                    "UNIQUE ("+Experiment.TIMESTAMP+","+Beacon.DEVICE_ID+")",
            Calibration_Readings_Base._ID + " integer primary key autoincrement,"
                    + Calibration_Readings_Base.TIMESTAMP + " real default 0,"
                    + Calibration_Readings_Base.DEVICE_ID + " text default '',"
                    + Calibration_Readings_Base.NUMBER_OF_READINGS + " integer default 0",
            Calibration_Readings._ID + " integer primary key autoincrement,"
                    + Calibration_Readings.BASE_ID  + " integer,"
                    + Calibration_Readings.TIMESTAMP + " real default 0,"
                    + Calibration_Readings.RSSI + " integer default 0,"
                    + Calibration_Readings.DISTANCE + " integer default 0,"
                    + " FOREIGN KEY (" + Calibration_Readings.BASE_ID + ") REFERENCES " +
                        "plugin_Calibration_Readings_base" + " (" + Calibration_Readings_Base._ID + ")"
    };

    static {
        uriMatcher = new UriMatcher(UriMatcher.NO_MATCH);
        uriMatcher.addURI(AUTHORITY, DATABASE_TABLES[0], BEACONS);
        uriMatcher.addURI(AUTHORITY, DATABASE_TABLES[0]+"/#", BEACONS_ID);
        uriMatcher.addURI(AUTHORITY, DATABASE_TABLES[1], EXPERIMENTS);
        uriMatcher.addURI(AUTHORITY, DATABASE_TABLES[1]+"/#", EXPERIMENTS_ID);
        uriMatcher.addURI(AUTHORITY, DATABASE_TABLES[2], CALIBRATION_READINGS_BASE);
        uriMatcher.addURI(AUTHORITY, DATABASE_TABLES[2] + "/#", CALIBRATION_READINGS_BASE_ID);
        uriMatcher.addURI(AUTHORITY, DATABASE_TABLES[3], CALIBRATION_READINGS);
        uriMatcher.addURI(AUTHORITY, DATABASE_TABLES[3] + "/#", CALIBRATION_READINGS_ID);

        beaconMap = new HashMap<>();
        beaconMap.put(Beacon._ID, Beacon._ID);
        beaconMap.put(Beacon.TIMESTAMP, Beacon.TIMESTAMP);
        beaconMap.put(Beacon.DEVICE_ID, Beacon.DEVICE_ID);
        beaconMap.put(Beacon.DEVICE_ADDRESS, Beacon.DEVICE_ADDRESS);
        beaconMap.put(Beacon.RSSI, Beacon.RSSI);
        beaconMap.put(Beacon.NUMBER_OF_BYTES_FOLLOWING_FIRST_AD_STRUCTURE, Beacon.NUMBER_OF_BYTES_FOLLOWING_FIRST_AD_STRUCTURE);
        beaconMap.put(Beacon.AD_TYPE, Beacon.AD_TYPE);
        beaconMap.put(Beacon.FLAG, Beacon.FLAG);
        beaconMap.put(Beacon.LE_LIMITED_DISCOVERABLE_MODE, Beacon.LE_LIMITED_DISCOVERABLE_MODE);
        beaconMap.put(Beacon.LE_GENERAL_DISCOVERABLE_MODE, Beacon.LE_GENERAL_DISCOVERABLE_MODE);
        beaconMap.put(Beacon.BR_EDR_NOT_SUPPORTED, Beacon.BR_EDR_NOT_SUPPORTED);
        beaconMap.put(Beacon.SIMULTANEOUS_LE_AND_BR_EDR_TO_SAME_DEVICE_CAPABLE_CONTROLLER, Beacon.SIMULTANEOUS_LE_AND_BR_EDR_TO_SAME_DEVICE_CAPABLE_CONTROLLER);
        beaconMap.put(Beacon.SIMULTANEOUS_LE_AND_BR_EDR_TO_SAME_DEVICE_CAPABLE_HOST, Beacon.SIMULTANEOUS_LE_AND_BR_EDR_TO_SAME_DEVICE_CAPABLE_HOST);
        beaconMap.put(Beacon.NUMBER_OF_BYTES_FOLLOWING_SECOND_AD_STRUCTURE, Beacon.NUMBER_OF_BYTES_FOLLOWING_SECOND_AD_STRUCTURE);
        beaconMap.put(Beacon.MANUFACTURER_SPECIFIC_DATA_AD_TYPE, Beacon.MANUFACTURER_SPECIFIC_DATA_AD_TYPE);
        beaconMap.put(Beacon.COMPANY_IDENTIFIER_CODE, Beacon.COMPANY_IDENTIFIER_CODE);
        beaconMap.put(Beacon.BYTE_0_OF_IBEACON_ADVERTISEMENT_INDICATOR, Beacon.BYTE_0_OF_IBEACON_ADVERTISEMENT_INDICATOR);
        beaconMap.put(Beacon.BYTE_1_OF_IBEACON_ADVERTISEMENT_INDICATOR, Beacon.BYTE_1_OF_IBEACON_ADVERTISEMENT_INDICATOR);
        beaconMap.put(Beacon.PROXIMITY_UUID, Beacon.PROXIMITY_UUID);
        beaconMap.put(Beacon.MAJOR, Beacon.MAJOR);
        beaconMap.put(Beacon.MINOR, Beacon.MINOR);
        beaconMap.put(Beacon.TWOS_COMPLEMENT_OF_CALIBRATED_TX_POWER, Beacon.TWOS_COMPLEMENT_OF_CALIBRATED_TX_POWER);
        beaconMap.put(Beacon.CHECKSUM, Beacon.CHECKSUM);
        beaconMap.put(Beacon.FORMATTED_PROXIMITY_UUID, Beacon.FORMATTED_PROXIMITY_UUID);
        beaconMap.put(Beacon.FORMATTED_MAJOR, Beacon.FORMATTED_MAJOR);
        beaconMap.put(Beacon.FORMATTED_MINOR, Beacon.FORMATTED_MINOR);

        experimentMap = new HashMap<>();
        experimentMap.put(Experiment._ID, Experiment._ID);
        experimentMap.put(Experiment.TIMESTAMP, Experiment.TIMESTAMP);
        experimentMap.put(Experiment.DEVICE_ID, Experiment.DEVICE_ID);
        experimentMap.put(Experiment.LABEL, Experiment.LABEL);
        experimentMap.put(Experiment.START, Experiment.START);
        experimentMap.put(Experiment.STOP, Experiment.STOP);

        calibrationReadingsMap = new HashMap<>();
        calibrationReadingsMap.put(Calibration_Readings.BASE_ID, Calibration_Readings.BASE_ID);
        calibrationReadingsMap.put(Calibration_Readings_Base.TIMESTAMP, Calibration_Readings_Base.TIMESTAMP);
        calibrationReadingsMap.put(Calibration_Readings_Base.DEVICE_ID, Calibration_Readings_Base.DEVICE_ID);
        calibrationReadingsMap.put(Calibration_Readings.RSSI, Calibration_Readings.RSSI);
        calibrationReadingsMap.put(Calibration_Readings.DISTANCE, Calibration_Readings.DISTANCE);

        calibrationReadingsBaseMap = new HashMap<>();
        calibrationReadingsBaseMap.put(Calibration_Readings_Base._ID, Calibration_Readings_Base._ID);
        calibrationReadingsBaseMap.put(Calibration_Readings_Base.TIMESTAMP, Calibration_Readings_Base.TIMESTAMP);
        calibrationReadingsBaseMap.put(Calibration_Readings_Base.DEVICE_ID, Calibration_Readings_Base.DEVICE_ID);
        calibrationReadingsBaseMap.put(Calibration_Readings_Base.NUMBER_OF_READINGS, Calibration_Readings_Base.NUMBER_OF_READINGS);
    }

    @Override
    public int delete(Uri uri, String selection, String[] selectionArgs) {
        if( database == null || ! database.isOpen()) database = databaseHelper.getWritableDatabase();

        int count;
        switch (uriMatcher.match(uri)) {
            case BEACONS:
                count = database.delete(DATABASE_TABLES[0], selection, selectionArgs);
                break;
            case EXPERIMENTS:
                count = database.delete(DATABASE_TABLES[1], selection, selectionArgs);
                break;
            case CALIBRATION_READINGS_BASE:
                count = database.delete(DATABASE_TABLES[2], selection, selectionArgs);
                break;
            case CALIBRATION_READINGS:
                count = database.delete(DATABASE_TABLES[3], selection, selectionArgs);
                break;
            default:
                throw new IllegalArgumentException("Unknown URI " + uri);
        }
        getContext().getContentResolver().notifyChange(uri, null);
        return count;
    }

    @Override
    public String getType(Uri uri) {
        switch (uriMatcher.match(uri)) {
            case BEACONS:
                return Beacon.CONTENT_TYPE;
            case BEACONS_ID:
                return Beacon.CONTENT_ITEM_TYPE;
            case EXPERIMENTS:
                return Experiment.CONTENT_TYPE;
            case EXPERIMENTS_ID:
                return Experiment.CONTENT_ITEM_TYPE;
            case CALIBRATION_READINGS_BASE:
                return Calibration_Readings_Base.CONTENT_TYPE;
            case CALIBRATION_READINGS_BASE_ID:
                return Calibration_Readings_Base.CONTENT_ITEM_TYPE;
            case CALIBRATION_READINGS:
                return Calibration_Readings.CONTENT_TYPE;
            case CALIBRATION_READINGS_ID:
                return Calibration_Readings.CONTENT_ITEM_TYPE;
            default:
                throw new IllegalArgumentException("Unknown URI " + uri);
        }
    }

    @Override
    public Uri insert(Uri uri, ContentValues initialValues) {

        if( database == null || ! database.isOpen()) database = databaseHelper.getWritableDatabase();

        ContentValues values = (initialValues != null) ? new ContentValues(initialValues) : new ContentValues();
        long _id;

        switch(uriMatcher.match(uri)) {
            case BEACONS:
                _id = database.insert(DATABASE_TABLES[0], Beacon.TIMESTAMP, values);
                if (_id > 0) {
                    Uri dataUri = ContentUris.withAppendedId(Beacon.CONTENT_URI, _id);
                    getContext().getContentResolver().notifyChange(dataUri, null);
                    return dataUri;
                }
                throw new SQLException("Failed to insert row into " + uri);
            case EXPERIMENTS:
                _id = database.insert(DATABASE_TABLES[1], null, values);
                if (_id > 0) {
                    Uri dataUri = ContentUris.withAppendedId(Experiment.CONTENT_URI, _id);
                    getContext().getContentResolver().notifyChange(dataUri, null);
                    return dataUri;
                }
                throw new SQLException("Failed to insert row into " + uri);
            case CALIBRATION_READINGS_BASE:
                _id = database.insert(DATABASE_TABLES[2], Calibration_Readings_Base.TIMESTAMP, values);
                if (_id > 0) {
                    Uri dataUri = ContentUris.withAppendedId(Calibration_Readings_Base.CONTENT_URI, _id);
                    getContext().getContentResolver().notifyChange(dataUri, null);
                    return dataUri;
                }
                throw new SQLException("Failed to insert row into " + uri);
            case CALIBRATION_READINGS:
                _id = database.insert(DATABASE_TABLES[3], Calibration_Readings.TIMESTAMP, values);
                if (_id > 0) {
                    Uri dataUri = ContentUris.withAppendedId(Calibration_Readings.CONTENT_URI, _id);
                    getContext().getContentResolver().notifyChange(dataUri, null);
                    return dataUri;
                }
                throw new SQLException("Failed to insert row into " + uri);
            default:
                throw new IllegalArgumentException("Unknown URI " + uri);
        }
    }

    @Override
    public boolean onCreate() {
        if( databaseHelper == null ) databaseHelper = new DatabaseHelper(getContext(), DATABASE_NAME, null, DATABASE_VERSION, DATABASE_TABLES, TABLES_FIELDS );
        database = databaseHelper.getWritableDatabase();
        return (databaseHelper != null);
    }

    @Override
    public Cursor query(Uri uri, String[] projection, String selection,
                        String[] selectionArgs, String sortOrder) {

        if( database == null || ! database.isOpen()) database = databaseHelper.getWritableDatabase();

        SQLiteQueryBuilder qb = new SQLiteQueryBuilder();
        switch (uriMatcher.match(uri)) {
            case BEACONS:
                qb.setTables(DATABASE_TABLES[0]);
                qb.setProjectionMap(beaconMap);
                break;
            case EXPERIMENTS:
                qb.setTables(DATABASE_TABLES[1]);
                qb.setProjectionMap(experimentMap);
                break;
            case CALIBRATION_READINGS_BASE:
                qb.setTables(DATABASE_TABLES[2]);
                qb.setProjectionMap(calibrationReadingsBaseMap);
                break;
            case CALIBRATION_READINGS:
                qb.setTables(DATABASE_TABLES[2] + " JOIN " + DATABASE_TABLES[3]
                        + " ON (" + DATABASE_TABLES[2] + "." + Calibration_Readings_Base._ID + " == "
                        + DATABASE_TABLES[3] + "." + Calibration_Readings.BASE_ID + ")");
                qb.setProjectionMap(calibrationReadingsMap);
                break;
            default:
                throw new IllegalArgumentException("Unknown URI " + uri);
        }
        try {
            Cursor c = qb.query(database, projection, selection, selectionArgs, null, null, sortOrder);
            c.setNotificationUri(getContext().getContentResolver(), uri);
            return c;
        }catch ( IllegalStateException e ) {
            if ( Aware.DEBUG ) Log.e(Aware.TAG, e.getMessage());
            return null;
        }
    }

    @Override
    public int update(Uri uri, ContentValues values, String selection,
                      String[] selectionArgs) {

        if( database == null || ! database.isOpen()) database = databaseHelper.getWritableDatabase();

        int count;
        switch (uriMatcher.match(uri)) {
            case BEACONS:
                count = database.update(DATABASE_TABLES[0], values, selection, selectionArgs);
                break;
            case EXPERIMENTS:
                count = database.update(DATABASE_TABLES[1], values, selection, selectionArgs);
                break;
            case CALIBRATION_READINGS_BASE:
                count = database.update(DATABASE_TABLES[2], values, selection, selectionArgs);
                break;
            case CALIBRATION_READINGS:
                count = database.update(DATABASE_TABLES[3], values, selection, selectionArgs);
                break;
            default:
                database.close();
                throw new IllegalArgumentException("Unknown URI " + uri);
        }
        getContext().getContentResolver().notifyChange(uri, null);
        return count;
    }
}
