package com.aware.plugin.beacons.micromap;

import android.app.AlertDialog;
import android.content.Context;
import android.graphics.*;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;

import com.aware.plugin.beacons.Constants;
import com.aware.plugin.beacons.Singleton;
import com.aware.plugin.beacons.model.AbstractBeacon;
import com.aware.plugin.beacons.semweb.JSONBeacon;
import com.aware.plugin.beacons.util.HexUtils;
import com.vividsolutions.jts.android.IdentityPointTransformation;
import com.vividsolutions.jts.android.PointTransformation;
import com.vividsolutions.jts.android.ShapeWriter;
import com.vividsolutions.jts.android.geom.DrawableShape;
import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Envelope;

import com.vividsolutions.jts.geom.util.AffineTransformation;
import pl.edu.agh.geist.micromaps.*;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

public class AreaVisualiser extends View implements View.OnTouchListener {
    private static final int MARGIN = 50;
    private static final int LEFT_MARGIN = 50;
    private static final int RIGHT_MARGIN = 140;
    private static final int LABEL_MARGIN_X = 10;
    private static final Paint backgroundPaint = new Paint();
    private static final Paint obstaclePaint = new Paint();
    private static final Paint accessibleFloorPaint = new Paint();
    private static final Paint beaconPaint = new Paint();
    private static final Paint textPaint = new Paint();
    private static final Paint closestBeaconPaint = new Paint();
    private static final Paint similarBeaconPaint = new Paint();

    static{
        backgroundPaint.setColor(Color.WHITE);
        obstaclePaint.setColor(Color.RED);
        accessibleFloorPaint.setColor(Color.LTGRAY);
        beaconPaint.setColor(Color.BLACK);
        textPaint.setColor(Color.BLACK);
        textPaint.setTextSize(30);
        closestBeaconPaint.setColor(Color.BLACK);
        closestBeaconPaint.setFakeBoldText(true);
        closestBeaconPaint.setTextSize(40);
        similarBeaconPaint.setColor(Color.BLACK);
        similarBeaconPaint.setTextSize(40);
    }

    private AffineTransformation affineTransform;
    private Area area;
    private LinkedList<User> users;
    private ShapeWriter sw;

    private Map<Beacon, Coordinate> beaconCoordinateMap;

    public AreaVisualiser(Context context, Area area, LinkedList<User> users) {
        super(context);
        setArea(area);
        setUsers(users);
        sw = new ShapeWriter(new TopologyTransform());
        beaconCoordinateMap = new HashMap<>();
        setFocusable(true);
        setFocusableInTouchMode(true);
        this.setOnTouchListener(this);
    }

    protected void onDraw(Canvas canvas) {

        setTransform();

        // Paint the area first
        DrawableShape shape = sw.toShape(area);
        shape.fill(canvas, backgroundPaint);
        shape.draw(canvas, backgroundPaint);

        //after that print the content of the area

        //physical objects
        LinkedList<Obstacle> objects = area.getObstacles();
        for(Obstacle po : objects){
            DrawableShape shapeObject = sw.toShape(po);
            shapeObject.fill(canvas, obstaclePaint);
            shapeObject.draw(canvas, obstaclePaint);
        }

        //draw AccessibleFloor
        DrawableShape floor = sw.toShape(area.getAccessibleFloor());
        floor.fill(canvas, accessibleFloorPaint);
        floor.draw(canvas, accessibleFloorPaint);

        //beacons
        LinkedList<Beacon> beacons = area.getBeacons();
        for(Beacon b : beacons){

            DrawableShape shapeObject = sw.toShape(b);
            shapeObject.fill(canvas, beaconPaint);
            shapeObject.draw(canvas, beaconPaint);

            RectF beaconRectF = ((com.vividsolutions.jts.android.geom.RectShape) shapeObject).getRectF();
            int beaconRectShapeCenterX = (int) (beaconRectF.left + (beaconRectF.width()/2));
            int beaconRectShapeCenterY = (int) (beaconRectF.top + (beaconRectF.height()/2));

            beaconCoordinateMap.put(b, new Coordinate(beaconRectShapeCenterX, beaconRectShapeCenterY));

            int majorInt = -1;
            int minorInt = -1;
            try{
                majorInt = Integer.parseInt(b.getMajor());
                minorInt = Integer.parseInt(b.getMinor());
            }catch (NumberFormatException ex1){
                try{
                    majorInt = Integer.decode("0x" + b.getMajor());
                    minorInt = Integer.decode("0x" + b.getMajor());
                }catch (NumberFormatException ex2){
                    Log.e(Constants.BEACONS_PLUGIN_TAG, ex2.getMessage());
                }
            }

            Paint paint = textPaint;
            AbstractBeacon closestBeacon;
            if((closestBeacon = Singleton.getInstance().getClosestBeacon()) != null &&
                    b.getUUID().equals(closestBeacon.getProximityUUID().toString().toUpperCase()) &&
                    majorInt == closestBeacon.getMajor() &&
                    minorInt == closestBeacon.getMinor()){
                paint = closestBeaconPaint;
            }else{
                for(JSONBeacon jsonBeacon : Singleton.getInstance().getSimilarBeacons()){
                    if(b.getUUID().equals(jsonBeacon.uuid) &&
                            majorInt == Integer.parseInt(jsonBeacon.major) &&
                            minorInt == Integer.parseInt(jsonBeacon.minor)){
                        paint = similarBeaconPaint;
                    }
                }
            }

            canvas.drawText("Beacon",
                    beaconRectShapeCenterX+LABEL_MARGIN_X,
                    beaconRectShapeCenterY,
                    paint);

//            canvas.drawText(b.getUUID(),
//                    beaconRectShapeCenterX+LABEL_MARGIN_X,
//                    beaconRectShapeCenterY-LABEL_MARGIN_Y,
//                    textPaint);
//
//            canvas.drawText("Major: "+b.getMajor(),
//                    beaconRectShapeCenterX+LABEL_MARGIN_X,
//                    beaconRectShapeCenterY,
//                    textPaint);
//
//            canvas.drawText("Minor: "+b.getMinor(),
//                    beaconRectShapeCenterX+LABEL_MARGIN_X,
//                    beaconRectShapeCenterY+LABEL_MARGIN_Y,
//                    textPaint);
        }
    }

    public boolean onTouch(View v, MotionEvent event) {
        if (event.getAction() == MotionEvent.ACTION_DOWN) {
            float x = event.getX();
            float y = event.getY();
            for (Map.Entry<Beacon, Coordinate> beaconCoordinateEntry : beaconCoordinateMap.entrySet()) {
                RectF beaconRectF = new RectF(
                        (int) beaconCoordinateEntry.getValue().getOrdinate(0) - 50,
                        (int) beaconCoordinateEntry.getValue().getOrdinate(1) - 50,
                        (int) beaconCoordinateEntry.getValue().getOrdinate(0) + 200,
                        (int) beaconCoordinateEntry.getValue().getOrdinate(1) + 50
                );
                if (beaconRectF.contains(x, y)) {
                    String majorString = beaconCoordinateEntry.getKey().getMajor();
                    String minorString = beaconCoordinateEntry.getKey().getMinor();
                    int majorInt = -1;
                    String majorHexString ="";
                    int minorInt = -1;
                    String minorHexString ="";

                    try{
                        majorInt = Integer.parseInt(majorString);
                        minorInt = Integer.parseInt(minorString);
                        majorHexString = HexUtils.intToHexString(majorInt);
                        minorHexString = HexUtils.intToHexString(minorInt);
                    }catch (NumberFormatException ex1){
                        try{
                            majorInt = Integer.decode("0x" + majorString);
                            minorInt = Integer.decode("0x" + minorString);
                            majorHexString = majorString;
                            minorHexString = minorString;
                        }catch (NumberFormatException ex2){
                            Log.e(Constants.BEACONS_PLUGIN_TAG, ex2.getMessage());
                        }
                    }
                    AlertDialog.Builder dialog = new AlertDialog.Builder(getContext());
                    dialog.setTitle("Beacon details");
                    dialog.setMessage("UUID: " + beaconCoordinateEntry.getKey().getUUID() + "\n"
                            + "Major: " + majorInt + " | " + majorHexString + "\n"
                            + "Minor: " + minorInt + " | " + minorHexString);
                    dialog.setNeutralButton("OK", null);
                    dialog.create().show();
                }
            }
        }
        return true;
    }

    private void setTransform() {
        Envelope env = getGeometryBounds();

        Rect visRect = new Rect();
        getGlobalVisibleRect(visRect);
        Rect drawingRect = new Rect(
                visRect.left + LEFT_MARGIN,
                visRect.top + MARGIN,
                visRect.right - RIGHT_MARGIN,
                visRect.bottom - MARGIN
        );
        double scale = Math.min(
                drawingRect.width() / env.getWidth(),
                drawingRect.height() / env.getHeight()
        );
        double xoff = LEFT_MARGIN - scale * env.getMinX();
        double yoff = MARGIN + env.getMaxY() * scale;

        affineTransform = new AffineTransformation(scale, 0, xoff, 0, -scale, yoff);
    }

    private Envelope getGeometryBounds() {
        Envelope env = new Envelope();
        Envelope geomEnv = area.getEnvelopeInternal();
        env.expandToInclude(geomEnv);
        return env;
    }

    /**
     * @return the users
     */
    public LinkedList<User> getUsers() {
        return users;
    }

    /**
     * @param users the users to set
     */
    public void setUsers(LinkedList<User> users) {
        this.users = users;
    }

    public Area getArea() {
        return area;
    }

    public void setArea(Area area) {
        this.area = area;
    }

    private class TopologyTransform implements PointTransformation{

        @Override
        public void transform(Coordinate toTransform, PointF stageTwoTransform) {
            IdentityPointTransformation ipt = new IdentityPointTransformation();
            PointF stageOneTransform = new PointF();
            ipt.transform(toTransform, stageOneTransform);
            Coordinate destCoordinate = new Coordinate();
            affineTransform.transform(new Coordinate(stageOneTransform.x, stageOneTransform.y), destCoordinate);
            stageTwoTransform.set((float) destCoordinate.x, (float) destCoordinate.y);
        }
    }
}