package com.aware.plugin.beacons.model;

import android.bluetooth.BluetoothDevice;
import android.os.Bundle;
import android.os.Parcel;
import android.os.Parcelable;
import com.kontakt.sdk.android.device.Beacon;

import java.util.UUID;

/**
 * Wrapper class around com.kontakt.sdk.android.device.Beacon.
 */
public class KontaktBeacon extends AbstractBeacon{

    private static final String PARCEL_SCAN_RESULT = "parcel_scan_result";
    private static final String PARCEL_BEACON = "parcel_beacon";
    private static final String PARCEL_LOG_CURVE_PARAMETER_A = "parcel_log_curve_parameter_A";
    private static final String PARCEL_LOG_CURVE_PARAMETER_B = "parcel_log_curve_parameter_b";
    private static final String PARCEL_LOG_CURVE_PARAMETER_N = "parcel_log_curve_parameter_n";
    private static final String PARCEL_DISTANCE = "parcel_distance";

    private Beacon beacon;

    public KontaktBeacon(final BluetoothDevice bluetoothDevice,
                         final int rssiValue,
                         final byte[] scanRecord,
                         ScanResult scanResult){
        beacon = new Beacon(bluetoothDevice, rssiValue, scanRecord);
        this.scanResult = scanResult;
    }

    public KontaktBeacon(Beacon beacon,
                         ScanResult scanResult){
        this.beacon = beacon;
        this.scanResult = scanResult;
    }

    public void setBeacon(Beacon beacon){
        this.beacon = beacon;
    }

    public static final Parcelable.Creator<KontaktBeacon> CREATOR = new Parcelable.Creator<KontaktBeacon>() {
        public KontaktBeacon createFromParcel(Parcel in) {
            return new KontaktBeacon(in);
        }

        public KontaktBeacon[] newArray(int size) {
            return new KontaktBeacon[size];
        }
    };

    private KontaktBeacon(Parcel input) {
        final Bundle bundle = input.readBundle(((Object)this).getClass().getClassLoader());
        beacon = bundle.getParcelable(PARCEL_BEACON);
        scanResult = bundle.getParcelable(PARCEL_SCAN_RESULT);
        logCurveParameter_A = bundle.getDouble(PARCEL_LOG_CURVE_PARAMETER_A);
        logCurveParameter_b = bundle.getDouble(PARCEL_LOG_CURVE_PARAMETER_B);
        logCurveParameter_n = bundle.getDouble(PARCEL_LOG_CURVE_PARAMETER_N);
        distance = bundle.getDouble(PARCEL_DISTANCE);
    }

    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel output, int flags) {

        final Bundle bundle = new Bundle(((Object)this).getClass().getClassLoader());
        bundle.putParcelable(PARCEL_BEACON, beacon);
        bundle.putParcelable(PARCEL_SCAN_RESULT, scanResult);
        bundle.putDouble(PARCEL_LOG_CURVE_PARAMETER_A, logCurveParameter_A);
        bundle.putDouble(PARCEL_LOG_CURVE_PARAMETER_B, logCurveParameter_b);
        bundle.putDouble(PARCEL_LOG_CURVE_PARAMETER_N, logCurveParameter_n);
        bundle.putDouble(PARCEL_DISTANCE, distance);

        output.writeBundle(bundle);
    }

    @Override
    public UUID getProximityUUID() {
        return beacon.getProximityUUID();
    }

    @Override
    public long getTimestamp() {
        return beacon.getTimestamp();
    }

    @Override
    public int getMajor() {
        return beacon.getMajor();
    }

    @Override
    public int getMinor() {
        return beacon.getMinor();
    }

    @Override
    public int getTxPower() {
        return beacon.getTxPower();
    }

    @Override
    public double getAccuracy() {
        return beacon.getAccuracy();
    }

    @Override
    public String getMacAddress() {
        return beacon.getMacAddress();
    }

    @Override
    public String getName() {
        return beacon.getName();
    }

    @Override
    public double getRssi() {
        return beacon.getRssi();
    }

    public Beacon getBeacon(){
        return beacon;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || ((Object)this).getClass() != o.getClass()) return false;

        KontaktBeacon that = (KontaktBeacon) o;

        if (!beacon.equals(that.beacon)) return false;

        return true;
    }

    @Override
    public int hashCode() {
        return beacon.hashCode();
    }
}
