package com.aware.plugin.beacons.model;

import android.os.Bundle;
import android.os.Parcel;
import android.os.Parcelable;

import java.util.UUID;

public class MockBeacon extends AbstractBeacon {

    private static final String PARCEL_NAME = "parcel_name";
    private static final String PARCEL_ADDRESS = "parcel_address";
    private static final String PARCEL_RSSI = "parcel_rssi";
    private static final String PARCEL_PROXIMITY_UUID = "parcel_proximity_uuid";
    private static final String PARCEL_MAJOR = "parcel_major";
    private static final String PARCEL_MINOR = "parcel_minor";
    private static final String PARCEL_TXPOWER = "parcel_txpower";
    private static final String PARCEL_ACCURACY = "parcel_accuracy";
    private static final String PARCEL_TIMESTAMP = "parcel_timestamp";
    private static final String PARCEL_SCAN_RESULT = "parcel_scan_result";
    private static final String PARCEL_LOG_CURVE_PARAMETER_A = "parcel_log_curve_parameter_A";
    private static final String PARCEL_LOG_CURVE_PARAMETER_B = "parcel_log_curve_parameter_b";
    private static final String PARCEL_LOG_CURVE_PARAMETER_N = "parcel_log_curve_parameter_n";
    private static final String PARCEL_DISTANCE = "parcel_distance";

    private int rssi;
    private UUID proximityUUID;
    private int major;
    private int minor;
    private int txPower;
    private String address;
    private double accuracy;
    private long timestamp;
    private String name;

    public MockBeacon(final int rssiValue, String address, String name, UUID proximityUUID,
                               int major, int minor, int txPower, double distance,
                               double logCurveParameter_A, double logCurveParameter_b, double logCurveParameter_n){
        this.rssi = rssiValue;
        this.proximityUUID = proximityUUID;
        this.major = major;
        this.minor = minor;
        this.txPower = txPower;

        if (rssi == 0) {
            accuracy = -1.0;
        }else {
            double ratio = rssi * 1.0 / txPower;
            if (ratio < 1.0) {
                accuracy = Math.pow(ratio, 10);
            } else {
                accuracy = (0.89976) * Math.pow(ratio, 7.7095) + 0.111;
            }
        }

        timestamp = System.currentTimeMillis();
        this.address = address;
        this.name = name;
        this.distance = distance;
        this.logCurveParameter_A = logCurveParameter_A;
        this.logCurveParameter_b = logCurveParameter_b;
        this.logCurveParameter_n = logCurveParameter_n;
    }

    /**
     * The Parcelable CREATOR constant.
     */
    public static final Parcelable.Creator<MockBeacon> CREATOR = new Parcelable.Creator<MockBeacon>() {
        public MockBeacon createFromParcel(Parcel in) {
            return new MockBeacon(in);
        }

        public MockBeacon[] newArray(int size) {
            return new MockBeacon[size];
        }
    };

    private MockBeacon(Parcel input) {
        final Bundle bundle = input.readBundle(((Object)this).getClass().getClassLoader());
        rssi = bundle.getInt(PARCEL_RSSI);
        name = bundle.getString(PARCEL_NAME);
        address = bundle.getString(PARCEL_ADDRESS);
        proximityUUID = (UUID) bundle.getSerializable(PARCEL_PROXIMITY_UUID);
        major = bundle.getInt(PARCEL_MAJOR);
        minor = bundle.getInt(PARCEL_MINOR);
        txPower = bundle.getInt(PARCEL_TXPOWER);
        accuracy = bundle.getDouble(PARCEL_ACCURACY);
        timestamp = bundle.getLong(PARCEL_TIMESTAMP);
        scanResult = bundle.getParcelable(PARCEL_SCAN_RESULT);
        logCurveParameter_A = bundle.getDouble(PARCEL_LOG_CURVE_PARAMETER_A);
        logCurveParameter_b = bundle.getDouble(PARCEL_LOG_CURVE_PARAMETER_B);
        logCurveParameter_n = bundle.getDouble(PARCEL_LOG_CURVE_PARAMETER_N);
        distance = bundle.getDouble(PARCEL_DISTANCE);
    }

    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel output, int flags) {

        final Bundle bundle = new Bundle(((Object)this).getClass().getClassLoader());
        bundle.putInt(PARCEL_RSSI, rssi);
        bundle.putString(PARCEL_NAME, name);
        bundle.putString(PARCEL_ADDRESS, address);
        bundle.putSerializable(PARCEL_PROXIMITY_UUID, proximityUUID);
        bundle.putInt(PARCEL_MAJOR, major);
        bundle.putInt(PARCEL_MINOR, minor);
        bundle.putInt(PARCEL_TXPOWER, txPower);
        bundle.putDouble(PARCEL_ACCURACY, accuracy);
        bundle.putLong(PARCEL_TIMESTAMP, timestamp);
        bundle.putParcelable(PARCEL_SCAN_RESULT, scanResult);
        bundle.putDouble(PARCEL_LOG_CURVE_PARAMETER_A, logCurveParameter_A);
        bundle.putDouble(PARCEL_LOG_CURVE_PARAMETER_B, logCurveParameter_b);
        bundle.putDouble(PARCEL_LOG_CURVE_PARAMETER_N, logCurveParameter_n);
        bundle.putDouble(PARCEL_DISTANCE, distance);

        output.writeBundle(bundle);
    }

    @Override
    public double getRssi() {
        return rssi;
    }

    @Override
    public UUID getProximityUUID() {
        return proximityUUID;
    }

    @Override
    public int getMajor() {
        return major;
    }

    @Override
    public int getMinor() {
        return minor;
    }

    @Override
    public int getTxPower() {
        return txPower;
    }

    @Override
    public String getMacAddress() {
        return address;
    }

    @Override
    public double getAccuracy() {
        return accuracy;
    }

    @Override
    public long getTimestamp() {
        return timestamp;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public byte getTwosComplementOfCalibratedTxPower(){
        return (byte) txPower;
    }

    @Override
    public int getScanResultRssi() {
        return rssi;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || ((Object)this).getClass() != o.getClass()) return false;

        MockBeacon that = (MockBeacon) o;

        if (major != that.major) return false;
        if (minor != that.minor) return false;
        if (!proximityUUID.equals(that.proximityUUID)) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = proximityUUID.hashCode();
        result = 31 * result + major;
        result = 31 * result + minor;
        return result;
    }

    public String toString(){
        return "Proximity UUID: " + getProximityUUID().toString().toUpperCase() +
                " | major: " + getMajor() +
                " | minor: " + getMinor() +
                " | rssi: " + rssi +
                " | " + Integer.toString(rssi & 0xFF) +
                " | distance: " + getDistance();
    }
}
