package com.aware.plugin.beacons.model;

import android.bluetooth.BluetoothDevice;
import android.os.Bundle;
import android.os.Parcel;
import android.os.Parcelable;

import java.nio.ByteBuffer;
import java.util.UUID;

public class OtherBeacon extends AbstractBeacon {

    private static final String PARCEL_DEVICE = "parcel_device";
    private static final String PARCEL_RSSI = "parcel_rssi";
    private static final String PARCEL_PROXIMITY_UUID = "parcel_proximity_uuid";
    private static final String PARCEL_MAJOR = "parcel_major";
    private static final String PARCEL_MINOR = "parcel_minor";
    private static final String PARCEL_TXPOWER = "parcel_txpower";
    private static final String PARCEL_ACCURACY = "parcel_accuracy";
    private static final String PARCEL_TIMESTAMP = "parcel_timestamp";
    private static final String PARCEL_SCAN_RESULT = "parcel_scan_result";
    private static final String PARCEL_LOG_CURVE_PARAMETER_A = "parcel_log_curve_parameter_A";
    private static final String PARCEL_LOG_CURVE_PARAMETER_B = "parcel_log_curve_parameter_b";
    private static final String PARCEL_LOG_CURVE_PARAMETER_N = "parcel_log_curve_parameter_n";
    private static final String PARCEL_DISTANCE = "parcel_distance";

    private BluetoothDevice device;
    private double rssi;
    private UUID proximityUUID;
    private int major;
    private int minor;
    private int txPower;
    private String address;
    private double accuracy;
    private long timestamp;
    private String name;

    public OtherBeacon(final BluetoothDevice bluetoothDevice,
                         final int rssiValue,
                         ScanResult scanResult){
        setBLEScanData(bluetoothDevice, rssiValue, scanResult);
    }

    public void setBLEScanData(final BluetoothDevice bluetoothDevice,
                                final int rssiValue,
                                ScanResult scanResult){
        this.scanResult = scanResult;
        this.rssi = rssiValue;

        final ByteBuffer byteBuffer = ByteBuffer.wrap(scanResult.getProximityUuid());
        proximityUUID = new UUID(byteBuffer.getLong(), byteBuffer.getLong());

        int uFirstByte = scanResult.getMajor()[0] & 0xFF;
        major = ((uFirstByte << 8) | (scanResult.getMajor()[1] & 0xFF));
        uFirstByte = scanResult.getMinor()[0] & 0xFF;
        minor = ((uFirstByte << 8) | (scanResult.getMinor()[1] & 0xFF));

        txPower = scanResult.getTwosComplementOfCalibratedTxPower();

        if (rssi == 0) {
            accuracy = -1.0;
        }else {
            double ratio = rssi * 1.0 / txPower;
            if (ratio < 1.0) {
                accuracy = Math.pow(ratio, 10);
            } else {
                accuracy = (0.89976) * Math.pow(ratio, 7.7095) + 0.111;
            }
        }

        timestamp = System.currentTimeMillis();
        device = bluetoothDevice;
        address = bluetoothDevice.getAddress();
        name = bluetoothDevice.getName();
    }

    /**
     * The Parcelable CREATOR constant.
     */
    public static final Parcelable.Creator<OtherBeacon> CREATOR = new Parcelable.Creator<OtherBeacon>() {
        public OtherBeacon createFromParcel(Parcel in) {
            return new OtherBeacon(in);
        }

        public OtherBeacon[] newArray(int size) {
            return new OtherBeacon[size];
        }
    };

    private OtherBeacon(Parcel input) {
        final Bundle bundle = input.readBundle(((Object)this).getClass().getClassLoader());
        rssi = bundle.getDouble(PARCEL_RSSI);
        device = bundle.getParcelable(PARCEL_DEVICE);
        name = device.getName();
        address = device.getAddress();
        proximityUUID = (UUID) bundle.getSerializable(PARCEL_PROXIMITY_UUID);
        major = bundle.getInt(PARCEL_MAJOR);
        minor = bundle.getInt(PARCEL_MINOR);
        txPower = bundle.getInt(PARCEL_TXPOWER);
        accuracy = bundle.getDouble(PARCEL_ACCURACY);
        timestamp = bundle.getLong(PARCEL_TIMESTAMP);
        scanResult = bundle.getParcelable(PARCEL_SCAN_RESULT);
        logCurveParameter_A = bundle.getDouble(PARCEL_LOG_CURVE_PARAMETER_A);
        logCurveParameter_b = bundle.getDouble(PARCEL_LOG_CURVE_PARAMETER_B);
        logCurveParameter_n = bundle.getDouble(PARCEL_LOG_CURVE_PARAMETER_N);
        distance = bundle.getDouble(PARCEL_DISTANCE);
    }

    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel output, int flags) {

        final Bundle bundle = new Bundle(((Object)this).getClass().getClassLoader());
        bundle.putDouble(PARCEL_RSSI, rssi);
        bundle.putParcelable(PARCEL_DEVICE, device);
        bundle.putSerializable(PARCEL_PROXIMITY_UUID, proximityUUID);
        bundle.putInt(PARCEL_MAJOR, major);
        bundle.putInt(PARCEL_MINOR, minor);
        bundle.putInt(PARCEL_TXPOWER, txPower);
        bundle.putDouble(PARCEL_ACCURACY, accuracy);
        bundle.putLong(PARCEL_TIMESTAMP, timestamp);
        bundle.putParcelable(PARCEL_SCAN_RESULT, scanResult);
        bundle.putDouble(PARCEL_LOG_CURVE_PARAMETER_A, logCurveParameter_A);
        bundle.putDouble(PARCEL_LOG_CURVE_PARAMETER_B, logCurveParameter_b);
        bundle.putDouble(PARCEL_LOG_CURVE_PARAMETER_N, logCurveParameter_n);
        bundle.putDouble(PARCEL_DISTANCE, distance);

        output.writeBundle(bundle);
    }

    @Override
    public double getRssi() {
        return rssi;
    }

    @Override
    public UUID getProximityUUID() {
        return proximityUUID;
    }

    @Override
    public int getMajor() {
        return major;
    }

    @Override
    public int getMinor() {
        return minor;
    }

    @Override
    public int getTxPower() {
        return txPower;
    }

    @Override
    public String getMacAddress() {
        return address;
    }

    @Override
    public double getAccuracy() {
        return accuracy;
    }

    @Override
    public long getTimestamp() {
        return timestamp;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || ((Object)this).getClass() != o.getClass()) return false;

        OtherBeacon that = (OtherBeacon) o;

        if (major != that.major) return false;
        if (minor != that.minor) return false;
        if (!proximityUUID.equals(that.proximityUUID)) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = proximityUUID.hashCode();
        result = 31 * result + major;
        result = 31 * result + minor;
        return result;
    }
}
