package com.aware.plugin.beacons.model;

import android.os.Parcel;
import android.os.Parcelable;
import com.aware.plugin.beacons.util.HexUtils;

import java.util.Arrays;

public class ScanResult implements Parcelable{

    private int rssi;

    private byte numberOfBytesFollowingFirstADStructure;
    private byte ADType;
    private byte flag;
    private boolean LELimitedDiscoverableMode;
    private boolean LEGeneralDiscoverableMode;
    private boolean BREDRNotSupported;
    private boolean simultaneousLEandBREDRtoSameDeviceCapableController;
    private boolean simultaneousLEandBREDRtoSameDeviceCapableHost;
    private byte numberOfBytesFollowingSecondADStructure;
    private byte manufacturerSpecificDataADType;
    private byte[] companyIdentifierCode;
    private byte byte0ofiBeaconAdvertisementIndicator;
    private byte byte1ofiBeaconAdvertisementIndicator;
    private byte[] proximityUuid;
    private byte[] major;
    private byte[] minor;
    private byte twosComplementOfCalibratedTxPower;
    private byte[] checksum;

    private String formattedProximityUuid;
    private String formattedMajor;
    private String formattedMinor;

    public ScanResult(int rssi) {
        this.rssi = rssi;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || o.getClass() != ScanResult.class) return false;

        ScanResult that = (ScanResult) o;

        if (!Arrays.equals(major, that.major)) return false;
        if (!Arrays.equals(minor, that.minor)) return false;
        if (!Arrays.equals(proximityUuid, that.proximityUuid)) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = Arrays.hashCode(proximityUuid);
        result = 31 * result + Arrays.hashCode(major);
        result = 31 * result + Arrays.hashCode(minor);
        return result;
    }

    public ScanResult(Parcel in){

        String[] data = new String[22];
        in.readStringArray(data);

        rssi = Integer.valueOf(data[0]);
        numberOfBytesFollowingFirstADStructure = Byte.valueOf(data[1]);
        ADType = Byte.valueOf(data[2]);
        flag = Byte.valueOf(data[3]);
        LELimitedDiscoverableMode = Boolean.valueOf(data[4]);
        LEGeneralDiscoverableMode = Boolean.valueOf(data[5]);
        BREDRNotSupported = Boolean.valueOf(data[6]);
        simultaneousLEandBREDRtoSameDeviceCapableController = Boolean.valueOf(data[7]);
        simultaneousLEandBREDRtoSameDeviceCapableHost = Boolean.valueOf(data[8]);
        numberOfBytesFollowingSecondADStructure = Byte.valueOf(data[9]);
        manufacturerSpecificDataADType = Byte.valueOf(data[10]);

        String[] s_companyIdentifierCode = data[11].substring(1, data[11].length() - 1).split(", ");
        companyIdentifierCode = new byte[s_companyIdentifierCode.length];
        for(int i = 0; i < s_companyIdentifierCode.length; i++){
            companyIdentifierCode[i] = Byte.valueOf(s_companyIdentifierCode[i]);
        }

        byte0ofiBeaconAdvertisementIndicator = Byte.valueOf(data[12]);
        byte1ofiBeaconAdvertisementIndicator = Byte.valueOf(data[13]);

        String[] s_proximityUuid = data[14].substring(1, data[14].length() - 1).split(", ");
        proximityUuid = new byte[s_proximityUuid.length];
        for(int i = 0; i < s_proximityUuid.length; i++){
            proximityUuid[i] = Byte.valueOf(s_proximityUuid[i]);
        }

        String[] s_major = data[15].substring(1, data[15].length() - 1).split(", ");
        major = new byte[s_major.length];
        for(int i = 0; i < s_major.length; i++){
            major[i] = Byte.valueOf(s_major[i]);
        }

        String[] s_minor = data[16].substring(1, data[16].length() - 1).split(", ");
        minor = new byte[s_minor.length];
        for(int i = 0; i < s_minor.length; i++){
            minor[i] = Byte.valueOf(s_minor[i]);
        }

        twosComplementOfCalibratedTxPower = Byte.valueOf(data[17]);

        String[] s_checksum = data[18].substring(1, data[18].length() - 1).split(", ");
        checksum = new byte[s_checksum.length];
        for(int i = 0; i < s_checksum.length; i++){
            checksum[i] = Byte.valueOf(s_checksum[i]);
        }

        formattedProximityUuid = data[19];
        formattedMajor = data[20];
        formattedMinor = data[21];
    }

    public int describeContents(){
        return 0;
    }

    @Override
    public void writeToParcel(Parcel dest, int flags) {
        dest.writeStringArray(new String[] {
                Integer.toString(this.rssi),
                Integer.toString(this.numberOfBytesFollowingFirstADStructure),
                Integer.toString(this.ADType),
                Integer.toString(this.flag),
                Boolean.toString(this.LELimitedDiscoverableMode),
                Boolean.toString(this.LEGeneralDiscoverableMode),
                Boolean.toString(this.BREDRNotSupported),
                Boolean.toString(this.simultaneousLEandBREDRtoSameDeviceCapableController),
                Boolean.toString(this.simultaneousLEandBREDRtoSameDeviceCapableHost),
                Integer.toString(this.numberOfBytesFollowingSecondADStructure),
                Integer.toString(this.manufacturerSpecificDataADType),
                Arrays.toString(this.companyIdentifierCode),
                Integer.toString(this.byte0ofiBeaconAdvertisementIndicator),
                Integer.toString(this.byte1ofiBeaconAdvertisementIndicator),
                Arrays.toString(this.proximityUuid),
                Arrays.toString(this.major),
                Arrays.toString(this.minor),
                Integer.toString(this.twosComplementOfCalibratedTxPower),
                Arrays.toString(this.checksum),
                formattedProximityUuid,
                formattedMajor,
                formattedMinor});
    }
    public static final Creator CREATOR = new Creator() {
        public ScanResult createFromParcel(Parcel in) {
            return new ScanResult(in);
        }

        public ScanResult[] newArray(int size) {
            return new ScanResult[size];
        }
    };

    @Override
    public String toString(){
        return "Proximity UUID: " + formattedProximityUuid + '\n' +
                "major: " + formattedMajor +
                " | minor: " + formattedMinor + " | rssi: " + rssi;
    }

    public String toFullString(){
        return "Proximity UUID: " + formattedProximityUuid + '\n' +
                "Major: " + formattedMajor + '\n' +
                "Minor: " + formattedMinor + '\n' +
                "RSSI: " + rssi + '\n' +
                "Number of bytes following first AD structure: " + HexUtils.intToHex(numberOfBytesFollowingFirstADStructure) +
                     " | " + numberOfBytesFollowingFirstADStructure + '\n' +
                "Flags AD type: " + HexUtils.intToHex(ADType) +
                     " | " + ADType + '\n' +
                "Flag: " + HexUtils.intToHex(flag) +
                     " | " + flag + '\n' +
                "LE limited discoverable mode: " + Boolean.toString(LELimitedDiscoverableMode) + '\n' +
                "LE general discoverable mode: " + Boolean.toString(LEGeneralDiscoverableMode) + '\n' +
                "BR/EDR Not Supported: " + Boolean.toString(BREDRNotSupported) + '\n' +
                "Simultaneous LE and BR/EDR to Same Device Capable (controller): " +
                    Boolean.toString(simultaneousLEandBREDRtoSameDeviceCapableController) + '\n' +
                "Simultaneous LE and BR/EDR to Same Device Capable (Host): " +
                    Boolean.toString(simultaneousLEandBREDRtoSameDeviceCapableHost) + '\n' +
                "Number of bytes following second AD structure: " + HexUtils.intToHex(numberOfBytesFollowingSecondADStructure) +
                    " | " + numberOfBytesFollowingSecondADStructure + '\n' +
                "Manufacturer specific data AD type: " + HexUtils.intToHex(manufacturerSpecificDataADType) +
                    " | " + manufacturerSpecificDataADType + '\n' +
                "Company identifier code: " + Arrays.toString(HexUtils.byteArrayToHexArray(companyIdentifierCode)) +
                    " | " + Arrays.toString(companyIdentifierCode) + '\n' +
                "Byte 0 of iBeacon advertisement indicator: " + HexUtils.intToHex(byte0ofiBeaconAdvertisementIndicator) +
                    " | " + byte0ofiBeaconAdvertisementIndicator + '\n' +
                "Byte 1 of iBeacon advertisement indicator: " + HexUtils.intToHex(byte1ofiBeaconAdvertisementIndicator) +
                    " | " + byte1ofiBeaconAdvertisementIndicator + '\n' +
                "The 2's complement of the calibrated Tx Power: " + HexUtils.intToHex(twosComplementOfCalibratedTxPower) +
                    " | " + twosComplementOfCalibratedTxPower + '\n' +
                "Checksum: " + Arrays.toString(HexUtils.byteArrayToHexArray(checksum)) +
                    " | " + Arrays.toString(checksum);
    }

    public int getRssi() {
        return rssi;
    }

    public void setRssi(int rssi) {
        this.rssi = rssi;
    }

    public byte getNumberOfBytesFollowingFirstADStructure() {
        return numberOfBytesFollowingFirstADStructure;
    }

    public void setNumberOfBytesFollowingFirstADStructure(byte numberOfBytesFollowingFirstADStructure) {
        this.numberOfBytesFollowingFirstADStructure = numberOfBytesFollowingFirstADStructure;
    }

    public byte getADType() {
        return ADType;
    }

    public void setADType(byte ADType) {
        this.ADType = ADType;
    }

    public byte getFlag() {
        return flag;
    }

    public void setFlag(byte flag) {
        this.flag = flag;
    }

    public boolean isLELimitedDiscoverableMode() {
        return LELimitedDiscoverableMode;
    }

    public void setLELimitedDiscoverableMode(boolean LELimitedDiscoverableMode) {
        this.LELimitedDiscoverableMode = LELimitedDiscoverableMode;
    }

    public boolean isLEGeneralDiscoverableMode() {
        return LEGeneralDiscoverableMode;
    }

    public void setLEGeneralDiscoverableMode(boolean LEGeneralDiscoverableMode) {
        this.LEGeneralDiscoverableMode = LEGeneralDiscoverableMode;
    }

    public boolean isBREDRNotSupported() {
        return BREDRNotSupported;
    }

    public void setBREDRNotSupported(boolean BREDRNotSupported) {
        this.BREDRNotSupported = BREDRNotSupported;
    }

    public boolean isSimultaneousLEandBREDRtoSameDeviceCapableController() {
        return simultaneousLEandBREDRtoSameDeviceCapableController;
    }

    public void setSimultaneousLEandBREDRtoSameDeviceCapableController(boolean simultaneousLEandBREDRtoSameDeviceCapableController) {
        this.simultaneousLEandBREDRtoSameDeviceCapableController = simultaneousLEandBREDRtoSameDeviceCapableController;
    }

    public boolean isSimultaneousLEandBREDRtoSameDeviceCapableHost() {
        return simultaneousLEandBREDRtoSameDeviceCapableHost;
    }

    public void setSimultaneousLEandBREDRtoSameDeviceCapableHost(boolean simultaneousLEandBREDRtoSameDeviceCapableHost) {
        this.simultaneousLEandBREDRtoSameDeviceCapableHost = simultaneousLEandBREDRtoSameDeviceCapableHost;
    }

    public byte getNumberOfBytesFollowingSecondADStructure() {
        return numberOfBytesFollowingSecondADStructure;
    }

    public void setNumberOfBytesFollowingSecondADStructure(byte numberOfBytesFollowingSecondADStructure) {
        this.numberOfBytesFollowingSecondADStructure = numberOfBytesFollowingSecondADStructure;
    }

    public byte getManufacturerSpecificDataADType() {
        return manufacturerSpecificDataADType;
    }

    public void setManufacturerSpecificDataADType(byte manufacturerSpecificDataADType) {
        this.manufacturerSpecificDataADType = manufacturerSpecificDataADType;
    }

    public byte[] getCompanyIdentifierCode() {
        return companyIdentifierCode;
    }

    public void setCompanyIdentifierCode(byte[] companyIdentifierCode) {
        this.companyIdentifierCode = companyIdentifierCode;
    }

    public byte getByte0ofiBeaconAdvertisementIndicator() {
        return byte0ofiBeaconAdvertisementIndicator;
    }

    public void setByte0ofiBeaconAdvertisementIndicator(byte byte0ofiBeaconAdvertisementIndicator) {
        this.byte0ofiBeaconAdvertisementIndicator = byte0ofiBeaconAdvertisementIndicator;
    }

    public byte getByte1ofiBeaconAdvertisementIndicator() {
        return byte1ofiBeaconAdvertisementIndicator;
    }

    public void setByte1ofiBeaconAdvertisementIndicator(byte byte1ofiBeaconAdvertisementIndicator) {
        this.byte1ofiBeaconAdvertisementIndicator = byte1ofiBeaconAdvertisementIndicator;
    }

    public byte[] getProximityUuid() {
        return proximityUuid;
    }

    public void setProximityUuid(byte[] proximityUuid) {
        this.proximityUuid = proximityUuid;
    }

    public byte[] getMajor() {
        return major;
    }

    public void setMajor(byte[] major) {
        this.major = major;
    }

    public byte[] getMinor() {
        return minor;
    }

    public void setMinor(byte[] minor) {
        this.minor = minor;
    }

    public byte getTwosComplementOfCalibratedTxPower() {
        return twosComplementOfCalibratedTxPower;
    }

    public void setTwosComplementOfCalibratedTxPower(byte twosComplementOfCalibratedTxPower) {
        this.twosComplementOfCalibratedTxPower = twosComplementOfCalibratedTxPower;
    }

    public byte[] getChecksum() {
        return checksum;
    }

    public void setChecksum(byte[] checksum) {
        this.checksum = checksum;
    }

    public String getFormattedProximityUuid() {
        return formattedProximityUuid;
    }

    public void setFormattedProximityUuid(String formattedProximityUuid) {
        this.formattedProximityUuid = formattedProximityUuid;
    }

    public String getFormattedMajor() {
        return formattedMajor;
    }

    public void setFormattedMajor(String formattedMajor) {
        this.formattedMajor = formattedMajor;
    }

    public String getFormattedMinor() {
        return formattedMinor;
    }

    public void setFormattedMinor(String formattedMinor) {
        this.formattedMinor = formattedMinor;
    }
}