package com.aware.plugin.beacons.parser;

import com.aware.plugin.beacons.model.ScanResult;
import com.aware.plugin.beacons.util.HexUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;

public class iBeaconPacketParser {

    private static final int COMPANY_IDENTIFIER_CODE_LENGTH = 2;
    private static final int PROXIMITY_UUID_LENGTH = 16;
    private static final int MAJOR_LENGTH = 2;
    private static final int MINOR_LENGTH = 2;
    private static final int CHECKSUM_LENGTH = 5;

    public static ScanResult parse(byte[] packet, int rssi) throws NotiBeaconException{

        ScanResult result = new ScanResult(rssi);

//        byte[] packet = toUnsigned(packet);

        ArrayList<Integer> buPacket = new ArrayList<>();
        for(int i : packet){
            buPacket.add(i);
        }

        if(findArray(buPacket) == -1){
            throw new NotiBeaconException("Required sequence of 4 bytes not present");
        }

        int j = 0;

        result.setNumberOfBytesFollowingFirstADStructure(packet[j++]);

        result.setADType(packet[j++]);

        byte flag = packet[j++];
        result.setFlag(flag);

        String flags = Integer.toBinaryString(flag);

        if(flags.length() < 5){
            StringBuilder builder = new StringBuilder();
            for(int i = 0; i < 5 - flags.length(); i++){
                builder.append('0');
            }
            builder.append(flags);
            flags = builder.toString();
        }

        result.setLELimitedDiscoverableMode(flags.charAt(4) == '1');
        result.setLEGeneralDiscoverableMode(flags.charAt(3) == '1');
        result.setBREDRNotSupported(flags.charAt(2) == '1');
        result.setSimultaneousLEandBREDRtoSameDeviceCapableController(flags.charAt(1) == '1');
        result.setSimultaneousLEandBREDRtoSameDeviceCapableHost(flags.charAt(0) == '1');

        result.setNumberOfBytesFollowingSecondADStructure(packet[j++]);
        result.setManufacturerSpecificDataADType(packet[j++]);

        byte[] companyIdentifierCode  = new byte[COMPANY_IDENTIFIER_CODE_LENGTH];
        for(int i = 0; i < COMPANY_IDENTIFIER_CODE_LENGTH; i++){
            companyIdentifierCode[i] = packet[j++];
        }
        result.setCompanyIdentifierCode(companyIdentifierCode);

        result.setByte0ofiBeaconAdvertisementIndicator(packet[j++]);

        result.setByte1ofiBeaconAdvertisementIndicator(packet[j++]);

        byte[] proximityUuid = new byte[PROXIMITY_UUID_LENGTH];
        for(int i = 0; i < PROXIMITY_UUID_LENGTH; i++){
            proximityUuid[i] = packet[j++];
        }
        result.setProximityUuid(proximityUuid);

        byte[] major = new byte[MAJOR_LENGTH];
        for(int i = 0; i < MAJOR_LENGTH; i++){
            major[i] = packet[j++];
        }
        result.setMajor(major);

        byte[] minor = new byte[MINOR_LENGTH];
        for(int i = 0; i < MINOR_LENGTH; i++){
            minor[i] = packet[j++];
        }
        result.setMinor(minor);

        result.setTwosComplementOfCalibratedTxPower(packet[j++]);

        byte[] checksum = new byte[CHECKSUM_LENGTH];
        for(int i = 0; i < CHECKSUM_LENGTH; i++){
            checksum[i] = packet[j++];
        }
        result.setChecksum(checksum);

        result.setFormattedProximityUuid(formatProximityUuid(HexUtils.byteArrayToHexArray(proximityUuid)));
        result.setFormattedMajor(HexUtils.byteToHex(major[0]) + HexUtils.byteToHex(major[1]));
        result.setFormattedMinor(HexUtils.byteToHex(minor[0]) + HexUtils.byteToHex(minor[1]));

        return result;
    }

//    private byte[] toUnsigned(byte[] bytes){
//        byte[] unsigned = new byte[bytes.length];
//        for(int j = 0; j < bytes.length; j++ ){
//            byte nb = bytes[j] & 0xFF;
//            unsigned[j] = nb;
//        }
//        return unsigned;
//    }

    private static int findArray(ArrayList<Integer> list)
    {
        Integer[] subArray = new Integer[]{0x4C, 0x00, 0x02, 0x15};
        return Collections.indexOfSubList(list, Arrays.asList(subArray));
    }

    private static String formatProximityUuid(String[] proximityUuid){

        final int FIRST_OFFSET_LENGTH = 4;
        final int SECOND_OFFSET_LENGTH = 2;
        final int THIRD_OFFSET_LENGTH = 2;
        final int FOURTH_OFFSET_LENGTH = 2;
        final int FIFTH_OFFSET_LENGTH = 6;

        int[] OFFSETS = new int[]{FIRST_OFFSET_LENGTH,
                SECOND_OFFSET_LENGTH,
                THIRD_OFFSET_LENGTH,
                FOURTH_OFFSET_LENGTH,
                FIFTH_OFFSET_LENGTH};

        StringBuilder builder = new StringBuilder();

        int k = 0;
        for(int i = 0; i < OFFSETS.length; i++){
            for(int j = 0; j < OFFSETS[i]; j++){
                builder.append(proximityUuid[k++]);
            }
            if(i < OFFSETS.length - 1){
                builder.append('-');
            }
        }

        return builder.toString();
    }
}