package com.aware.plugin.beacons.semweb;

import android.util.Log;
import android.view.View;
import android.widget.Toast;
import com.aware.plugin.beacons.Constants;
import com.aware.plugin.beacons.Singleton;
import com.aware.plugin.beacons.model.AbstractBeacon;
import com.aware.plugin.beacons.view.MapFragment;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

public class SimilarBeaconsRunnable implements Runnable {

    private MapFragment parent;

    public SimilarBeaconsRunnable(MapFragment parent){
        this.parent = parent;
    }

    public void run() {

        AbstractBeacon closestBeacon = Singleton.getInstance().getClosestBeacon();

        final Set<JSONBeacon> similarBeacons = new HashSet<>();
        JSONBeacon inputBeacon = new JSONBeacon(
                closestBeacon.getProximityUUID().toString().toUpperCase(),
                Integer.toString(closestBeacon.getMajor()),
                Integer.toString(closestBeacon.getMinor()));
        similarBeacons.add(inputBeacon);

        try {
            String tagQueryJSONResponse = FusekiAPICall.executeSPARQLQuery(
                    parent.getActivity(),
                    SPARQLQueryCreator.tagQuery(inputBeacon.uuid, inputBeacon.major, inputBeacon.minor)
            );

            JSONArray tagQueryBindings = (new JSONObject(tagQueryJSONResponse))
                    .getJSONObject("results")
                    .getJSONArray("bindings");

            for (int i = 0; i < tagQueryBindings.length(); i++) {

                JSONArray beaconsQueryBindings = (
                        new JSONObject(
                                FusekiAPICall.executeSPARQLQuery(
                                        parent.getActivity(),
                                        SPARQLQueryCreator.beaconsQuery(
                                                (String) tagQueryBindings.getJSONObject(i)
                                                        .getJSONObject("poitag")
                                                        .get("value"))
                                )
                        )
                ).getJSONObject("results").getJSONArray("bindings");

                for (int j = 0; j < beaconsQueryBindings.length(); j++) {
                    similarBeacons.add(new JSONBeacon(
                                    (String) beaconsQueryBindings.getJSONObject(j)
                                            .getJSONObject("uuid").get("value"),
                                    (String) beaconsQueryBindings.getJSONObject(j)
                                            .getJSONObject("major").get("value"),
                                    (String) beaconsQueryBindings.getJSONObject(j)
                                            .getJSONObject("minor").get("value"))
                    );
                }

                similarBeacons.remove(inputBeacon);

                Singleton.getInstance().setSimilarBeacons(new ArrayList<>(similarBeacons));
                View view = parent.getView();
                if (view != null) {
                    view.post(new Runnable() {
                        public void run() {
                            parent.redrawMap();
                        }
                    });
                }
            }

        } catch (JSONException e) {
            Log.d(Constants.BEACONS_PLUGIN_TAG, e.getMessage());
        }catch (java.net.ConnectException e) {
            Log.d(Constants.BEACONS_PLUGIN_TAG, e.getMessage());
            View view = parent.getView();
            if (view != null) {
                view.post(new Runnable() {
                    public void run() {
                        Toast.makeText(parent.getActivity(), "Could not connect", Toast.LENGTH_SHORT).show();
                    }
                });
            }
        }catch (IOException e) {
            Log.d(Constants.BEACONS_PLUGIN_TAG, e.getMessage());
        }
    }
}