package com.aware.plugin.beacons.view;

import android.graphics.Color;
import android.support.v7.app.ActionBarActivity;
import android.view.Menu;
import android.view.MenuItem;
import com.aware.plugin.beacons.R;
import lecho.lib.hellocharts.gesture.ZoomType;
import lecho.lib.hellocharts.model.*;
import lecho.lib.hellocharts.view.LineChartView;

import java.util.List;

public abstract class AbstractHelloChartActivity extends ActionBarActivity {

    protected List<List<PointValue>> chartData;
    protected LineChartView chart;
    protected boolean hasAxes = true;
    protected boolean hasAxesNames = true;
    protected boolean hasLines = true;
    protected boolean hasPoints = true;
    protected ValueShape shape = ValueShape.CIRCLE;
    protected boolean isFilled = false;
    protected boolean hasLabels = false;
    protected boolean isCubic = false;
    protected boolean hasLabelForSelected = false;
    protected int[] colors = {Color.RED, Color.GREEN, Color.BLUE, Color.DKGRAY};
    protected int right;

    protected abstract void prepareChartData();

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.line_chart_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch(item.getItemId()) {
            case android.R.id.home:
                onBackPressed();
                return true;
            case R.id.action_reset:
                reset();
                generateData();
                return true;
            case R.id.action_toggle_lines:
                toggleLines();
                return true;
            case R.id.action_toggle_points:
                togglePoints();
                return true;
            case R.id.action_shape_circles:
                setCircles();
                return true;
            case R.id.action_shape_square:
                setSquares();
                return true;
            case R.id.action_toggle_labels:
                toggleLabels();
                return true;
            case R.id.action_toggle_axes:
                toggleAxes();
                return true;
            case R.id.action_toggle_axes_names:
                toggleAxesNames();
                return true;
            case R.id.action_toggle_selection_mode:
                toggleLabelForSelected();
//                Toast.makeText(this,
//                        "Selection mode set to " + chart.isValueSelectionEnabled() + " select any point.",
//                        Toast.LENGTH_SHORT).show();
                return true;
            case R.id.action_toggle_touch_zoom:
                chart.setZoomEnabled(!chart.isZoomEnabled());
//                Toast.makeText(this, "IsZoomEnabled " + chart.isZoomEnabled(), Toast.LENGTH_SHORT).show();
                return true;
            case R.id.action_zoom_both:
                chart.setZoomType(ZoomType.HORIZONTAL_AND_VERTICAL);
                return true;
            case R.id.action_zoom_horizontal:
                chart.setZoomType(ZoomType.HORIZONTAL);
                return true;
            case R.id.action_zoom_vertical:
                chart.setZoomType(ZoomType.VERTICAL);
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    protected float findLowest(List<List<PointValue>> lines){
        float lowest = 0;
        for(List<PointValue> line : lines){
            for(PointValue value : line){
                if(value.getY() < lowest){
                    lowest = value.getY();
                }
            }
        }
        return lowest;
    }

    protected float findHighest(List<List<PointValue>> lines){
        float highest = -150;
        for(List<PointValue> line : lines){
            for(PointValue value : line){
                if(value.getY() > highest){
                    highest = value.getY();
                }
            }
        }
        return highest;
    }

    protected void reset() {
        hasAxes = true;
        hasAxesNames = true;
        hasLines = true;
        hasPoints = true;
        shape = ValueShape.CIRCLE;
        isFilled = false;
        hasLabels = false;
        isCubic = false;
        hasLabelForSelected = false;
        chart.setValueSelectionEnabled(false);
        resetViewport();
    }

    protected void resetViewport() {
        final Viewport v = new Viewport(chart.getMaximumViewport());
        v.bottom = findLowest(chartData);
        v.top = findHighest(chartData);
        v.left = 0;
        v.right = right;
        chart.setMaximumViewport(v);
        chart.setCurrentViewport(v, false);
    }

    protected abstract void generateData();

    protected void toggleLines() {
        hasLines = !hasLines;
        generateData();
    }

    protected void togglePoints() {
        hasPoints = !hasPoints;
        generateData();
    }

    protected void setCircles() {
        shape = ValueShape.CIRCLE;
        generateData();
    }

    protected void setSquares() {
        shape = ValueShape.SQUARE;
        generateData();
    }

    protected void toggleLabels() {
        hasLabels = !hasLabels;
        if (hasLabels) {
            hasLabelForSelected = false;
            chart.setValueSelectionEnabled(false);
        }
        generateData();
    }

    protected void toggleLabelForSelected() {
        hasLabelForSelected = !hasLabelForSelected;
        chart.setValueSelectionEnabled(hasLabelForSelected);
        if (hasLabelForSelected) {
            hasLabels = false;
        }
        generateData();
    }

    protected void toggleAxes() {
        hasAxes = !hasAxes;
        generateData();
    }

    protected void toggleAxesNames() {
        hasAxesNames = !hasAxesNames;
        generateData();
    }
}
