package com.aware.plugin.beacons.view;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.support.v7.app.ActionBarActivity;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;
import com.aware.plugin.beacons.R;
import com.aware.plugin.beacons.Singleton;
import com.aware.plugin.beacons.model.AbstractBeacon;
import com.aware.plugin.beacons.model.KontaktBeacon;
import com.aware.plugin.beacons.util.HexUtils;

import java.text.DecimalFormat;

import static com.aware.plugin.beacons.Constants.BEACON_EXTRA;
import static com.aware.plugin.beacons.Constants.SCAN_RESULT_ACTION;

public class BeaconDetailsActivity extends ActionBarActivity {

    private AbstractBeacon beacon;

    private TextView deviceRSSITextView;
    private TextView deviceAccuracyTextView;
    private TextView deviceDistanceTextView;

    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.beacon_details_activity_layout);

        AbstractBeacon abstractBeacon = getIntent().getParcelableExtra(BEACON_EXTRA);
        beacon = Singleton.getInstance().getExistingBeacon(abstractBeacon);

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        getSupportActionBar().setTitle("Beacon details");
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setBackgroundDrawable(new ColorDrawable(Color.rgb(8, 120, 165)));

        TextView deviceAddressTextView = (TextView) findViewById(R.id.device_address);
        deviceAddressTextView.setText(beacon.getMacAddress());

        TextView deviceProximityUuidTextView = (TextView) findViewById(R.id.device_proximity_uuid);
        deviceProximityUuidTextView.setText(beacon.getProximityUUID().toString().toUpperCase());

        TextView deviceMajorDecTextView = (TextView) findViewById(R.id.device_major_dec);
        deviceMajorDecTextView.setText(Integer.toString(beacon.getMajor()));

        TextView deviceMajorHexTextView = (TextView) findViewById(R.id.device_major_hex);
        deviceMajorHexTextView.setText(HexUtils.intToHexString(beacon.getMajor()));

        TextView deviceMinorDecTextView = (TextView) findViewById(R.id.device_minor_dec);
        deviceMinorDecTextView.setText(Integer.toString(beacon.getMinor()));

        TextView deviceMinorHexTextView = (TextView) findViewById(R.id.device_minor_hex);
        deviceMinorHexTextView.setText(HexUtils.intToHexString(beacon.getMinor()));

        deviceRSSITextView = (TextView) findViewById(R.id.device_rssi);
//        deviceRSSITextView.setText(Double.toString(Math.round(beacon.getRssi())));
        deviceRSSITextView.setText(beacon.getScanResultRssi() +
                " | " + (beacon.getScanResultRssi() & 0xFF ));

        deviceAccuracyTextView = (TextView) findViewById(R.id.device_accuracy);
        deviceAccuracyTextView.setText(new DecimalFormat("#.##").format(beacon.getAccuracy()));

        TextView deviceSignalPowerTextView = (TextView) findViewById(R.id.device_tx_power);
        deviceSignalPowerTextView.setText(Byte.toString(beacon.getTwosComplementOfCalibratedTxPower()));

        deviceDistanceTextView = (TextView) findViewById(R.id.device_distance);
        deviceDistanceTextView.setText(new DecimalFormat("#.##").format(beacon.getDistance()));
    }

    @Override
    public void onResume() {
        super.onResume();
        registerReceiver(resultBroadcastReceiver, new IntentFilter(SCAN_RESULT_ACTION));
    }

    @Override
    public void onPause() {
        super.onPause();
        unregisterReceiver(resultBroadcastReceiver);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.beacon_details_menu, menu);
        if (beacon instanceof KontaktBeacon){
            menu.findItem(R.id.menu_connect).setVisible(true);
        }else{
            menu.findItem(R.id.menu_connect).setVisible(false);
        }
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        final Intent intent;
        switch(item.getItemId()) {
            case R.id.menu_connect:
                intent = new Intent(this, KontaktDeviceControlActivity.class);
                intent.putExtra(BEACON_EXTRA, beacon);
                startActivity(intent);
                return true;

            case R.id.menu_settings:
                intent = new Intent(this, SettingsActivity.class);
                intent.putExtra(BEACON_EXTRA, beacon);
                startActivity(intent);
                return true;

            case android.R.id.home:
                onBackPressed();
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    private BroadcastReceiver resultBroadcastReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            AbstractBeacon beacon = intent.getParcelableExtra(BEACON_EXTRA);

            if(beacon != null){
                if((BeaconDetailsActivity.this.beacon).equals(beacon)) {
                    BeaconDetailsActivity.this.beacon = beacon;

                    deviceRSSITextView.setText(beacon.getScanResultRssi() +
                            " | " + (beacon.getScanResultRssi() & 0xFF ));
                    deviceAccuracyTextView.setText(new DecimalFormat("#.##").format(beacon.getAccuracy()));
                    deviceDistanceTextView.setText(new DecimalFormat("#.##").format(beacon.getDistance()));

                    View v = findViewById(R.id.beacon_details_activity_layout);
                    v.invalidate();
                }
            }
        }
    };
}