package com.aware.plugin.beacons.view;

import android.content.ContentValues;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.*;
import com.aware.Aware;
import com.aware.Aware_Preferences;
import com.aware.plugin.beacons.R;
import com.aware.plugin.beacons.Singleton;
import com.aware.plugin.beacons.db.Beacons_Provider;
import com.aware.plugin.beacons.model.AbstractBeacon;

import java.util.ArrayList;

import static com.aware.plugin.beacons.Constants.*;

public class BeaconListFragment extends Fragment {

    private boolean DEBUG;
    private LeScanResultsListAdapter leScanResultsListAdapter;
    private BeaconScanBroadcastReceiverUI resultBroadcastReceiver;
    private long btnStartTimestamp = 0;

    public BeaconListFragment() {
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {

        DEBUG = Aware.getSetting(getActivity().getContentResolver(), Aware_Preferences.DEBUG_FLAG).equals("true");

        final View rootView = inflater.inflate(R.layout.beacons_list_fragment_layout, container, false);

        leScanResultsListAdapter = new LeScanResultsListAdapter(getActivity(),
                R.layout.list_item,
                android.R.id.text1,
                new ArrayList<>(Singleton.getInstance().getFoundDevices()));

        ListView listViewBleScanResults = (ListView) rootView.findViewById(R.id.listViewBleScanResults);
        listViewBleScanResults.setAdapter(leScanResultsListAdapter);
        listViewBleScanResults.setClickable(true);
        listViewBleScanResults.setOnItemClickListener(mMessageClickedHandler);

        resultBroadcastReceiver = new BeaconScanBroadcastReceiverUI(this);

        final Button btnStart = (Button) rootView.findViewById(R.id.btnStart);
        btnStart.setEnabled(true);

        final Button btnAdd = (Button) rootView.findViewById(R.id.btnAdd);
        btnAdd.setEnabled(false);

        btnStart.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                btnStart.setEnabled(false);
                btnStartTimestamp = System.currentTimeMillis();
                btnAdd.setEnabled(true);
            }
        });

        btnAdd.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                btnStart.setEnabled(true);
                String label = ((EditText)rootView.findViewById(R.id.editTextExperimentLabel)).getText().toString();

                ContentValues rowData = new ContentValues();

                long stopTimestamp = System.currentTimeMillis();

                rowData.put(Beacons_Provider.Experiment.TIMESTAMP, stopTimestamp);
                rowData.put(Beacons_Provider.Experiment.DEVICE_ID,
                        Aware.getSetting(getActivity().getContentResolver(), Aware_Preferences.DEVICE_ID));
                rowData.put(Beacons_Provider.Experiment.LABEL, label);
                rowData.put(Beacons_Provider.Experiment.START, btnStartTimestamp);
                rowData.put(Beacons_Provider.Experiment.STOP, stopTimestamp);

                getActivity().getContentResolver().insert(Beacons_Provider.Experiment.CONTENT_URI, rowData);

                if(DEBUG){
                    Log.d(BEACONS_PLUGIN_TAG, rowData.toString());
                    Log.d(Aware.TAG, rowData.toString());
                }

                Intent share = new Intent(ACTION_AWARE_BEACONS);
                getActivity().sendBroadcast(share);

                btnAdd.setEnabled(false);
            }
        });

        return rootView;
    }

    @Override
    public void onResume() {
        super.onResume();
        getActivity().registerReceiver(resultBroadcastReceiver, new IntentFilter(SCAN_RESULT_ACTION));
    }

    @Override
    public void onPause() {
        super.onPause();
        getActivity().unregisterReceiver(resultBroadcastReceiver);
    }

    private AdapterView.OnItemClickListener mMessageClickedHandler = new AdapterView.OnItemClickListener() {
        public void onItemClick(AdapterView parent, View v, int position, long id) {

            final AbstractBeacon beacon = leScanResultsListAdapter.getBeacons().get(position);

            Intent intent = new Intent(getActivity(), BeaconDetailsActivity.class);
            intent.putExtra(BEACON_EXTRA, beacon);
            getActivity().startActivity(intent);
        }
    };

    public void updateUI(){
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                leScanResultsListAdapter.notifyDataSetChanged();
            }
        });
    }

    public LeScanResultsListAdapter getLeScanResultsListAdapter() {
        return leScanResultsListAdapter;
    }
}
