package com.aware.plugin.beacons.view;

import android.annotation.SuppressLint;
import android.app.ProgressDialog;
import android.content.*;
import android.content.pm.ActivityInfo;
import android.graphics.Color;
import android.graphics.Point;
import android.graphics.drawable.ColorDrawable;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.support.v7.app.ActionBarActivity;
import android.util.Log;
import android.view.*;
import android.widget.*;
import com.aware.Aware;
import com.aware.Aware_Preferences;
import com.aware.plugin.beacons.Constants;
import com.aware.plugin.beacons.R;
import com.aware.plugin.beacons.Settings;
import com.aware.plugin.beacons.Singleton;
import com.aware.plugin.beacons.db.Beacons_Provider;
import com.aware.plugin.beacons.model.AbstractBeacon;
import com.aware.plugin.beacons.model.CalibrationReading;

import java.text.SimpleDateFormat;
import java.util.*;

import static com.aware.plugin.beacons.Constants.*;

public class CalibrationProcessActivity extends ActionBarActivity {

    private static final String BUNDLE_STEP = "STEP";
    private static final String BUNDLE_MAC = "MAC";
    private static final String BUNDLE_TEXT = "TEXT";
    private static final String BUNDLE_FINISHED = "FINISHED";
    private static final String BUNDLE_TIMER = "TIMER";
    private static final String BUNDLE_TIMER_TIMESTAMP = "TIMER_TIMESTAMP";
    private static final String BUNDLE_RECEIVING = "RECEIVING";

    private boolean DEBUG;
    private String macAddress;
    private TextView readingsTextView;
    private ScrollView readingsScrollView;
    private TextView timer;
    private Handler handler = new Handler();
    private int stepCounter;
    private int timeWindowSeconds;
    private int timerCounter;
    private long timerTimestamp;
    private boolean isReceiving;
    private boolean isFinished;
    private String readingsText;

    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        timeWindowSeconds =
                Integer.valueOf(PreferenceManager.getDefaultSharedPreferences(this)
                        .getString(CALIBRATION_PROCESS_STEP_TIME, "30"));

        DEBUG = Aware.getSetting(getContentResolver(), Aware_Preferences.DEBUG_FLAG).equals("true");

        stepCounter = savedInstanceState == null ? -1 : savedInstanceState.getInt(BUNDLE_STEP);
        setContent();
        setStepSpecificStuff();

        if(savedInstanceState != null){
            if(stepCounter >= 0 && stepCounter < 3) {

                readingsText = savedInstanceState.getString(BUNDLE_TEXT);
                readingsTextView.setText(readingsText);

                if(isFinished = savedInstanceState.getBoolean(BUNDLE_FINISHED)){
                    setContinueStuffVisibility(View.VISIBLE);
                    timer.setText("0:00");
                }else{
                    setContinueStuffVisibility(View.INVISIBLE);
                }

                if(isReceiving = savedInstanceState.getBoolean(BUNDLE_RECEIVING)) {
                    registerReceiver(resultBroadcastReceiver, new IntentFilter(SCAN_RESULT_ACTION));

                    timerCounter = savedInstanceState.getInt(BUNDLE_TIMER);
                            timerTimestamp = savedInstanceState.getLong(BUNDLE_TIMER_TIMESTAMP);

                    long diff = System.currentTimeMillis() - timerTimestamp;

                    timerTimestamp = System.currentTimeMillis();

                    if (diff > 1000) {
                        handler.postDelayed(timerRunnable, 0);
                        handler.postDelayed(collectionRunnable, (timerCounter - 1) * 1000);
                    } else {
                        handler.postDelayed(timerRunnable, 1000 - diff);
                        handler.postDelayed(collectionRunnable, (1000 - diff) + (timerCounter - 1) * 1000);
                    }
                }
            }
            macAddress = savedInstanceState.getString(BUNDLE_MAC);
        }else{
            macAddress = getIntent().getStringExtra(BEACON_MAC_ADDRESS_EXTRA);
        }
    }

    private void setContent() {
        switch (stepCounter) {
            case -1:
                setContentView(R.layout.calibration_start_layout);
                findViewById(R.id.calibrationStartButton).setOnClickListener(new View.OnClickListener() {
                    @SuppressLint("UseSparseArrays")
                    @Override
                    public void onClick(View v) {
                        stepCounter++;
                        Singleton.getInstance().setCalibrationData(new HashMap<Integer, List<CalibrationReading>>());
                        setContent();
                        startCollecting();
                    }
                });
                break;
            case 0:
            case 1:
            case 2:
                setContentView(R.layout.calibration_data_collection_layout);
                readingsTextView = (TextView) findViewById(R.id.readingsTextView);
                readingsScrollView = (ScrollView) findViewById(R.id.readingsScrollView);
                readingsScrollView.setVerticalScrollBarEnabled(true);
                timer = (TextView) findViewById(R.id.timer);
                break;
        }
        android.support.v7.widget.Toolbar toolbar = (android.support.v7.widget.Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        getSupportActionBar().setTitle("Calibration process");
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setBackgroundDrawable(new ColorDrawable(Color.rgb(8, 120, 165)));
    }

    private void setStepSpecificStuff() {
        switch (stepCounter) {
            case 0:
                ((ImageView) findViewById(R.id.distanceImageView)).setImageResource(R.drawable.zero_meters_calibration);
                ((TextView) findViewById(R.id.calibrationFurtherDistanceInstructions)).setText(R.string.calibration_further_distance_instructions);
                findViewById(R.id.calibrationContinueButton).setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        stepCounter++;
                        startCollecting();
                    }
                });
                break;
            case 1:
                ((ImageView) findViewById(R.id.distanceImageView)).setImageResource(R.drawable.one_meter_calibration);
                ((TextView) findViewById(R.id.calibrationFurtherDistanceInstructions)).setText("Please move 2 meters away from the beacon.");
                findViewById(R.id.calibrationContinueButton).setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        stepCounter++;
                        startCollecting();
                    }
                });
                break;
            case 2:
                ((ImageView) findViewById(R.id.distanceImageView)).setImageResource(R.drawable.two_meters_calibration);
                ((TextView) findViewById(R.id.calibrationFurtherDistanceInstructions)).setText("The calibration sequence is complete.");
                ((Button)findViewById(R.id.calibrationContinueButton)).setText("End");
                findViewById(R.id.calibrationContinueButton).setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        lockOrientation();
                        launchRingDialog();
                    }
                });
                break;
        }
    }

    public void launchRingDialog() {
        final ProgressDialog ringProgressDialog = ProgressDialog.show(CalibrationProcessActivity.this,
                "Please wait ...",
                "Saving calibration data ...",
                true);
        ringProgressDialog.setCancelable(false);
        new Thread(new Runnable() {
            @SuppressLint("UseSparseArrays")
            @Override
            public void run() {
                try {
                    ContentValues data = new ContentValues();
                    data.put(Beacons_Provider.Calibration_Readings_Base.TIMESTAMP, System.currentTimeMillis());
                    data.put(Beacons_Provider.Calibration_Readings_Base.DEVICE_ID, Aware.getSetting(getContentResolver(), Aware_Preferences.DEVICE_ID));

                    int num = Singleton.getInstance().getCalibrationDataCount();
                    Log.d(Constants.BEACONS_PLUGIN_TAG, "Number of readings: " + num);

                    data.put(Beacons_Provider.Calibration_Readings_Base.NUMBER_OF_READINGS, num);

                    Uri uri = getContentResolver().insert(Beacons_Provider.Calibration_Readings_Base.CONTENT_URI, data);

                    long base_id = ContentUris.parseId(uri);

                    Log.d(Constants.BEACONS_PLUGIN_TAG, "Base id: " + base_id);

                    for (Map.Entry<Integer, List<CalibrationReading>> e : Singleton.getInstance().getCalibrationData().entrySet()) {
                        for (CalibrationReading reading : e.getValue()) {
                            ContentValues rowData = new ContentValues();
                            rowData.put(Beacons_Provider.Calibration_Readings.BASE_ID, base_id);
                            rowData.put(Beacons_Provider.Calibration_Readings.TIMESTAMP, reading.getTimestamp());
                            rowData.put(Beacons_Provider.Calibration_Readings.RSSI, reading.getRssi());
                            rowData.put(Beacons_Provider.Calibration_Readings.DISTANCE, e.getKey());
                            Uri u = getContentResolver().insert(Beacons_Provider.Calibration_Readings.CONTENT_URI, rowData);
                            Log.d(Constants.BEACONS_PLUGIN_TAG, "Inserted reading id: " + ContentUris.parseId(u));
                        }
                    }

                    Intent share = new Intent(ACTION_AWARE_CALIBRATION_READING);
                    sendBroadcast(share);

                    Intent intent = new Intent(CalibrationProcessActivity.this, Settings.class);
                    intent.putExtra(CALIBRATION_FRAGMENT_EXTRA, true);
                    intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP);
                    startActivity(intent);
                }catch(Exception ex){
                    Log.e(Constants.BEACONS_PLUGIN_TAG, ex.getMessage());
                }

                ringProgressDialog.dismiss();
                Singleton.getInstance().setCalibrationData(new HashMap<Integer, List<CalibrationReading>>());
                CalibrationProcessActivity.this.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_UNSPECIFIED);
            }
        }).start();
    }

    private void setContinueStuffVisibility(int visibility){
        findViewById(R.id.calibrationFurtherDistanceInstructions).setVisibility(visibility);
        findViewById(R.id.calibrationReadyContinueMessage).setVisibility(visibility);
        findViewById(R.id.calibrationContinueButton).setVisibility(visibility);
    }

    private Runnable timerRunnable = new Runnable() {
        @Override
        public void run() {
            if(timerCounter > 0){
                timerCounter--;
            }
            timer.setText(new SimpleDateFormat("m:ss").format(timerCounter * 1000));
            if (timerCounter > 0) {
                handler.postDelayed(this, 1000);
            }
        }
    };

    private Runnable collectionRunnable = new Runnable() {
        @Override
        public void run() {
            timerTimestamp = -1;
            stopCollecting();
        }
    };

    private BroadcastReceiver resultBroadcastReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            AbstractBeacon beacon = intent.getParcelableExtra(BEACON_EXTRA);
            if(beacon.getMacAddress().equals(macAddress)){
                Singleton.getInstance().addCalibrationReading(stepCounter, new CalibrationReading(beacon.getRssi(), beacon.getTimestamp()));

                readingsTextView.setText(
                        readingsText += new SimpleDateFormat("HH:mm:ss.SSS").format(new Date(beacon.getTimestamp())) +
                                " | RSSI : " +
                                beacon.getRssi() +
                                "\n"
                );
                readingsScrollView.fullScroll(View.FOCUS_DOWN);
            }
        }
    };

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putInt(BUNDLE_STEP, stepCounter);
        outState.putString(BUNDLE_MAC, macAddress);
        outState.putString(BUNDLE_TEXT, readingsText);
        outState.putBoolean(BUNDLE_FINISHED, isFinished);
        if(isReceiving) {
            handler.removeCallbacks(timerRunnable);
            handler.removeCallbacks(collectionRunnable);
            outState.putInt(BUNDLE_TIMER, timerCounter);
            outState.putLong(BUNDLE_TIMER_TIMESTAMP, timerTimestamp);
            outState.putBoolean(BUNDLE_RECEIVING, isReceiving);
            try{
                unregisterReceiver(resultBroadcastReceiver);
            }catch (IllegalArgumentException ex){
                Log.e(Constants.BEACONS_PLUGIN_TAG, ex.getMessage());
            }
        }
    }

    private void startCollecting(){
        setStepSpecificStuff();
        setContinueStuffVisibility(View.INVISIBLE);
        readingsTextView.setText("");
        readingsText = "";
        isReceiving = true;
        isFinished = false;
        registerReceiver(resultBroadcastReceiver, new IntentFilter(SCAN_RESULT_ACTION));

        timerCounter = timeWindowSeconds;
        timer.setText(new SimpleDateFormat("m:ss").format(timeWindowSeconds * 1000));
        handler.postDelayed(timerRunnable , 1000);
        timerTimestamp = System.currentTimeMillis();
        handler.postDelayed(collectionRunnable, timeWindowSeconds * 1000);
    }

    private void stopCollecting(){
        setContinueStuffVisibility(View.VISIBLE);
        isReceiving = false;
        isFinished = true;
        try{
            unregisterReceiver(resultBroadcastReceiver);
        }catch (IllegalArgumentException ex){
            Log.e(Constants.BEACONS_PLUGIN_TAG, ex.getMessage());
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.empty_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch(item.getItemId()) {
            case android.R.id.home:
                onBackPressed();
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    /*
    http://stackoverflow.com/a/16599549
     */
    @SuppressWarnings("deprecation")
    @SuppressLint("NewApi")
    private void lockOrientation() {
        Display display = getWindowManager().getDefaultDisplay();
        int rotation = display.getRotation();
        int height;
        int width;
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.HONEYCOMB_MR2) {
            height = display.getHeight();
            width = display.getWidth();
        } else {
            Point size = new Point();
            display.getSize(size);
            height = size.y;
            width = size.x;
        }
        switch (rotation) {
            case Surface.ROTATION_90:
                if (width > height)
                    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
                else
                    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_REVERSE_PORTRAIT);
                break;
            case Surface.ROTATION_180:
                if (height > width)
                    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_REVERSE_PORTRAIT);
                else
                    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_REVERSE_LANDSCAPE);
                break;
            case Surface.ROTATION_270:
                if (width > height)
                    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_REVERSE_LANDSCAPE);
                else
                    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
                break;
            default :
                if (height > width)
                    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
                else
                    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
        }
    }
}
