package com.aware.plugin.beacons.view;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.support.v7.app.ActionBarActivity;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import com.aware.plugin.beacons.R;
import com.aware.plugin.beacons.Singleton;
import com.aware.plugin.beacons.db.Beacons_Provider;
import com.aware.plugin.beacons.model.CalibrationReading;

import java.util.HashMap;
import java.util.List;

import static com.aware.plugin.beacons.Constants.BASE_ID_EXTRA;

public class CalibrationProcessResultsActivity extends ActionBarActivity {

    @SuppressLint("UseSparseArrays")
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.calibration_charts_layout);

        int baseId = getIntent().getIntExtra(BASE_ID_EXTRA, 0);

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        getSupportActionBar().setTitle("Calibration charts");
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setBackgroundDrawable(new ColorDrawable(Color.rgb(8, 120, 165)));

        findViewById(R.id.btnShowRegularHelloChart).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                showRegularHelloChart();
            }
        });

        findViewById(R.id.btnShowLogCurveHelloChart).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                showLogCurveHelloChart();
            }
        });

        Cursor readingsCursor = getContentResolver().query(Beacons_Provider.Calibration_Readings.CONTENT_URI,
                new String[]{"plugin_beacons_calibrationreadings.distance as distance",
                        "plugin_beacons_calibrationreadings.rssi as rssi",
                        "plugin_beacons_calibrationreadings.timestamp as timestamp"},
                "BASE_ID = ?", new String[]{Integer.toString(baseId)},
                Beacons_Provider.Calibration_Readings.TIMESTAMP + " DESC");

        if(readingsCursor != null && readingsCursor.moveToFirst()){
            Singleton.getInstance().setCalibrationData(new HashMap<Integer, List<CalibrationReading>>());
            do{
                int distance = readingsCursor.getInt(readingsCursor.getColumnIndex(
                        Beacons_Provider.Calibration_Readings.DISTANCE));

                int rssi = readingsCursor.getInt(readingsCursor.getColumnIndex(
                        Beacons_Provider.Calibration_Readings.RSSI));

                long timestamp = (long) readingsCursor.getDouble(readingsCursor.getColumnIndex(
                        Beacons_Provider.Calibration_Readings.TIMESTAMP));

                Singleton.getInstance().addCalibrationReading(distance, new CalibrationReading(rssi, timestamp));

            }while(readingsCursor.moveToNext());
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.empty_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch(item.getItemId()) {
            case android.R.id.home:
                onBackPressed();
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    private void showRegularHelloChart(){
        Intent intent = new Intent(this, RegularHelloChartActivity.class);
        startActivity(intent);
    }

    private void showLogCurveHelloChart(){
        Intent intent = new Intent(this, LogCurveHelloChartActivity.class);
        startActivity(intent);
    }
}
