package com.aware.plugin.beacons.view;

import android.app.ListFragment;
import android.content.Intent;
import android.database.Cursor;
import android.database.sqlite.SQLiteException;
import android.os.Bundle;
import android.util.Log;
import android.view.*;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import com.aware.plugin.beacons.Constants;
import com.aware.plugin.beacons.R;
import com.aware.plugin.beacons.db.Beacons_Provider;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import static com.aware.plugin.beacons.Constants.BASE_ID_EXTRA;

public class CalibrationReadingsListFragment extends ListFragment {

    private List<Integer> baseIds;

    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        setHasOptionsMenu(true);
    }

    @Override
    public void onResume() {
        super.onResume();

        List<String> values = new ArrayList<>();

        try {
            Cursor calibrationReadingsBaseCursor = getActivity().getContentResolver().query(Beacons_Provider.Calibration_Readings_Base.CONTENT_URI, null,
                    null,
                    null,
                    Beacons_Provider.Calibration_Readings_Base.TIMESTAMP + " DESC");

            baseIds = new ArrayList<>();

            if (calibrationReadingsBaseCursor != null && calibrationReadingsBaseCursor.moveToFirst()) {
                while (!calibrationReadingsBaseCursor.isAfterLast()) {
                    int baseId = calibrationReadingsBaseCursor.getInt(
                            calibrationReadingsBaseCursor.getColumnIndex(Beacons_Provider.Calibration_Readings_Base._ID)
                    );
                    long timestamp = (long) calibrationReadingsBaseCursor.getDouble(
                            calibrationReadingsBaseCursor.getColumnIndex(Beacons_Provider.Calibration_Readings_Base.TIMESTAMP)
                    );
                    int numOfReadings = calibrationReadingsBaseCursor.getInt(
                            calibrationReadingsBaseCursor.getColumnIndex(Beacons_Provider.Calibration_Readings_Base.NUMBER_OF_READINGS)
                    );
                    values.add("ID: " + baseId +
                            " | " + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(timestamp) +
                            " | Num: " + numOfReadings);
                    baseIds.add(baseId);

                    calibrationReadingsBaseCursor.moveToNext();
                }
            }
        }catch (SQLiteException ex){
            Log.e(Constants.BEACONS_PLUGIN_TAG, ex.getMessage());
        }

        ArrayAdapter<String> adapter = new ArrayAdapter<>(getActivity(), android.R.layout.simple_list_item_1, values);
        setListAdapter(adapter);

        setEmptyText("There are no calibration readings in the database yet.");
    }

    @Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        Intent intent = new Intent(getActivity(), CalibrationProcessResultsActivity.class);
        intent.putExtra(BASE_ID_EXTRA, baseIds.get(position));
        getActivity().startActivity(intent);
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater){
        inflater.inflate(R.menu.calibration_readings_list_menu, menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch(item.getItemId()) {
            case R.id.menu_new_readings:
                Intent intent = new Intent(getActivity(), CalibrationBeaconSelectionActivity.class);
                getActivity().startActivity(intent);
                return true;
        }
        return super.onOptionsItemSelected(item);
    }
}
