package com.aware.plugin.beacons.view;

import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.preference.EditTextPreference;
import android.preference.PreferenceManager;
import android.util.AttributeSet;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.TextView;
import com.aware.plugin.beacons.R;
import com.aware.plugin.beacons.model.AbstractBeacon;

public class EditTextPreferenceWithValueLocal extends EditTextPreferenceWithValue {
    private TextView textValue;

    private String globalValue;
    private String localValue;

    private AbstractBeacon beacon;

    public EditTextPreferenceWithValueLocal(Context context) {
        super(context);
        SharedPreferences sharedPref = PreferenceManager.getDefaultSharedPreferences(context);
        globalValue = sharedPref.getString(getKey(), "");
    }

    public EditTextPreferenceWithValueLocal(Context context, AttributeSet attrs) {
        super(context, attrs);
        SharedPreferences sharedPref = PreferenceManager.getDefaultSharedPreferences(context);
        globalValue = sharedPref.getString(getKey(), "");
    }

    public EditTextPreferenceWithValueLocal(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        SharedPreferences sharedPref = PreferenceManager.getDefaultSharedPreferences(context);
        globalValue = sharedPref.getString(getKey(), "");
    }

    @Override
    protected void onBindView(View view) {
        super.onBindView(view);
        textValue = (TextView) view.findViewById(R.id.pref_value);
        if (textValue != null) {
            textValue.setText(localValue);
        }
    }

    @Override
    public void setText(String text) {
        super.setText(text);
        if (textValue != null) {
            textValue.setText(getText());
            localValue = text;
            double value = Double.valueOf(text);
            if(value != 0) {
                switch (getKey()) {
                    case "local_log_curve_parameter_a":
                        beacon.setLogCurveParameter_A(Double.valueOf(textValue.getText().toString()));
                        break;
                    case "local_log_curve_parameter_b":
                        beacon.setLogCurveParameter_b(Double.valueOf(textValue.getText().toString()));
                        break;
                    case "local_log_curve_parameter_n":
                        beacon.setLogCurveParameter_n(Double.valueOf(textValue.getText().toString()));
                        break;
                }
            }
        }
        persistString(globalValue);
    }

    public void setBeacon(AbstractBeacon beacon){
        this.beacon = beacon;
        switch(getKey()){
            case "local_log_curve_parameter_a":
                localValue = Double.toString(beacon.getLogCurveParameter_A());
                break;
            case "local_log_curve_parameter_b":
                localValue = Double.toString(beacon.getLogCurveParameter_b());
                break;
            case "local_log_curve_parameter_n":
                localValue = Double.toString(beacon.getLogCurveParameter_n());
                break;
        }
    }
}
