package com.aware.plugin.beacons.view;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.support.v7.app.ActionBarActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.*;
import com.aware.Aware;
import com.aware.Aware_Preferences;
import com.aware.plugin.beacons.Constants;
import com.aware.plugin.beacons.R;
import com.aware.plugin.beacons.model.KontaktBeacon;
import com.aware.plugin.beacons.util.AssetsPropertyReader;
import com.kontakt.sdk.android.device.Beacon;
import com.kontakt.sdk.android.manager.BeaconConnection;

import java.util.*;

import static com.aware.plugin.beacons.Constants.BEACON_EXTRA;

public class KontaktDeviceControlActivity extends ActionBarActivity {

    private static final String LIST_NAME_KEY = "NAME";
    private static final String LIST_DESC_KEY = "DESC";

    private boolean DEBUG = false;

    private KontaktBeacon beacon;
    private BeaconConnection connection;
    private Beacon.Characteristics characteristics;

    private ExpandableListView kontaktBeaconServicesList;
    private TextView mConnectionState;
    private TextView mAuthenticationState;

    private ArrayList<HashMap<String, String>> groupData = new ArrayList<>();
    private ArrayList<ArrayList<HashMap<String, String>>> childData = new ArrayList<>();

    private boolean mConnected = false;
    private boolean mAuthenticated = false;
    private boolean mConnecting = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.kontakt_beacon_control_activity_layout);

        DEBUG = Aware.getSetting(this.getContentResolver(), Aware_Preferences.DEBUG_FLAG).equals("true");

        final Intent intent = getIntent();
        beacon = intent.getParcelableExtra(BEACON_EXTRA);

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        getSupportActionBar().setTitle("Beacon control");
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        getSupportActionBar().setBackgroundDrawable(new ColorDrawable(Color.rgb(8, 120, 165)));

        kontaktBeaconServicesList = (ExpandableListView) findViewById(R.id.kontakt_beacon_services_list);
        kontaktBeaconServicesList.setOnChildClickListener(servicesListClickListener);
        mConnectionState = (TextView) findViewById(R.id.connection_state);
        mAuthenticationState = (TextView) findViewById(R.id.authetication_state);
        ((TextView) findViewById(R.id.device_address)).setText(beacon.getMacAddress());

        setUpList();

        connect();
    }

    @Override
    protected void onDestroy() {
        if(mConnected){
            disconnect();
        }
        super.onDestroy();
    }

    private void setUpList(){

        HashMap<String, String> readableGroupItemData = new HashMap<>();
        readableGroupItemData.put(LIST_NAME_KEY, "Readable");
        readableGroupItemData.put(LIST_DESC_KEY, "Readable services");
        groupData.add(readableGroupItemData);

        HashMap<String, String> writableGroupItemData = new HashMap<>();
        writableGroupItemData.put(LIST_NAME_KEY, "Writable");
        writableGroupItemData.put(LIST_DESC_KEY, "Writable services");
        groupData.add(writableGroupItemData);

        childData.add(loadServicePropertiesIntoList("kontaktreadableservices.properties"));
        childData.add(loadServicePropertiesIntoList("kontaktwritableservices.properties"));

        SimpleExpandableListAdapter gattServiceAdapter = new SimpleExpandableListAdapter(
                this,
                groupData, //gattServiceData
                android.R.layout.simple_expandable_list_item_2,
                new String[] {LIST_NAME_KEY, LIST_DESC_KEY},
                new int[] { android.R.id.text1, android.R.id.text2 },
                childData, //gattCharacteristicData
                android.R.layout.simple_expandable_list_item_2,
                new String[] {LIST_NAME_KEY, LIST_DESC_KEY},
                new int[] { android.R.id.text1, android.R.id.text2 }
        );

        kontaktBeaconServicesList.setAdapter(gattServiceAdapter);
    }

    private ArrayList<HashMap<String, String>> loadServicePropertiesIntoList(String propertiesFileName){
        AssetsPropertyReader assetsPropertyReader = new AssetsPropertyReader(this);
        Properties properties = assetsPropertyReader.getProperties(propertiesFileName);

        ArrayList<HashMap<String, String>> groupData = new ArrayList<>();
        Enumeration<Object> e = properties.keys();
        while (e.hasMoreElements()) {
            String s = (String)e.nextElement();
            HashMap<String, String> hm = new HashMap<>();
            hm.put(LIST_NAME_KEY, s);
            hm.put(LIST_DESC_KEY, properties.getProperty(s));
            groupData.add(hm);
        }
        return groupData;
    }

    private void askForPass(){
        AlertDialog.Builder alert = new AlertDialog.Builder(KontaktDeviceControlActivity.this);
        alert.setTitle("Password");
        alert.setMessage("Please input password for beacon " + beacon.getMacAddress());
        final EditText input = new EditText(KontaktDeviceControlActivity.this);
        alert.setView(input);
        alert.setPositiveButton("Ok", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
                beacon.getBeacon().setPassword(input.getText().toString().getBytes());
                connect();
            }
        });
        alert.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
                // Canceled.
            }
        });
        alert.show();
    }

    private void connect(){
        if(DEBUG){
            Log.d(Constants.BEACONS_PLUGIN_TAG, "connect()");
        }
        if(beacon.getBeacon().getPassword() == null || beacon.getBeacon().getPassword().length != 4){
            askForPass();
        }else {
            mConnecting = true;
            invalidateOptionsMenu();
            connection = BeaconConnection.newInstance(KontaktDeviceControlActivity.this,
                    beacon.getBeacon(), listener);
            if (connection.connect()) {
                if(DEBUG){
                    Log.d(Constants.BEACONS_PLUGIN_TAG, "connection successful");
                }
            } else {
                if(DEBUG){
                    Log.d(Constants.BEACONS_PLUGIN_TAG, "connection unsuccessful");
                }
            }
        }
    }

    private void disconnect(){
        if(DEBUG){
            Log.d(Constants.BEACONS_PLUGIN_TAG, "disconnect()");
        }
        connection.disconnect();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.beacon_control_menu, menu);
        if (mConnecting) {
            MenuItem connectMenuItem = menu.findItem(R.id.menu_connect);
            connectMenuItem.setVisible(true);
            connectMenuItem.setEnabled(false);
            connectMenuItem.setIcon(R.drawable.ic_action_bluetooth_searching);

            menu.findItem(R.id.menu_disconnect).setVisible(false);
        } else if (mConnected) {
            menu.findItem(R.id.menu_connect).setVisible(false);
            menu.findItem(R.id.menu_disconnect).setVisible(true);
        } else {
            MenuItem connectMenuItem = menu.findItem(R.id.menu_connect);
            connectMenuItem.setVisible(true);
            connectMenuItem.setEnabled(true);
            connectMenuItem.setIcon(R.drawable.ic_action_bluetooth_light);

            menu.findItem(R.id.menu_disconnect).setVisible(false);
        }
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch(item.getItemId()) {
            case R.id.menu_connect:
                connect();
                return true;
            case R.id.menu_disconnect:
                disconnect();
                return true;
            case R.id.menu_reset:
                if(connection != null){
                    askForReset();
                }
                return true;
            case android.R.id.home:
                onBackPressed();
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    private final ExpandableListView.OnChildClickListener servicesListClickListener =
            new ExpandableListView.OnChildClickListener() {
                @Override
                public boolean onChildClick(ExpandableListView parent, View v, int groupPosition, int childPosition, long id) {

                    if(!mAuthenticated){
                        Toast.makeText(KontaktDeviceControlActivity.this, "You are not authenticated!", Toast.LENGTH_SHORT).show();
                        return true;
                    }

                    String name = childData.get(groupPosition).get(childPosition).get(LIST_NAME_KEY);

                    switch(groupPosition){
                        case 0:
                            String value = "null";
                            switch (name){
                                case "Proximity UUID":
                                    value = characteristics.getProximityUUID().toString();
                                    break;
                                case "Major":
                                    value = Integer.toString(characteristics.getMajor());
                                    break;
                                case "Minor":
                                    value = Integer.toString(characteristics.getMinor());
                                    break;
                                case "Power Level":
                                    value = Integer.toString(characteristics.getPowerLevel());
                                    break;
                                case "Advertising Interval":
                                    value = Long.toString(characteristics.getAdvertisingInterval());
                                    break;
                                case "Battery Level":
                                    value = characteristics.getBatteryLevel();
                                    break;
                                case "Manufacturer Name":
                                    value = characteristics.getManufacturerName();
                                    break;
                                case "Model Name":
                                    value = characteristics.getModelName();
                                    break;
                                case "Firmware Revision":
                                    value = characteristics.getFirmwareRevision();
                                    break;
                                case "Hardware Revision":
                                    value = characteristics.getHardwareRevision();
                                    break;
                            }
                            showReadableCharacteristicInAlert(name, value);
                            break;
                        case 1:
                            switch (name) {
                                case "Reset Device":
                                    askForReset();
                                    break;
                                default:
                                    askForNewValueInAlert(name);
                                    break;
                            }
                            break;
                    }
                    return true;
                }
            };

    private void showReadableCharacteristicInAlert(String name, String value){
        final AlertDialog.Builder alert = new AlertDialog.Builder(KontaktDeviceControlActivity.this);
        alert.setTitle(name);
        alert.setMessage("The value for characteristic " + name + " is:\n" + value);
        alert.setNeutralButton("Close", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
            }
        });
        alert.show();
    }

    private void askForNewValueInAlert(final String name){
        AlertDialog.Builder alert = new AlertDialog.Builder(KontaktDeviceControlActivity.this);
        alert.setTitle(name);
        alert.setMessage("Please input new value for characteristic " + name + " :");
        final EditText input = new EditText(KontaktDeviceControlActivity.this);
        alert.setView(input);
        alert.setPositiveButton("Ok", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
                writeNewValue(name, input.getText().toString());
            }
        });
        alert.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
            }
        });
        alert.show();
    }

    private void askForReset(){
        if(!mAuthenticated){
            Toast.makeText(KontaktDeviceControlActivity.this, "You are not authenticated!", Toast.LENGTH_SHORT).show();
            return;
        }
        AlertDialog.Builder alert = new AlertDialog.Builder(KontaktDeviceControlActivity.this);
        alert.setTitle("Reset confirmation");
        alert.setMessage("Are you sure you want to reset the device?");
        alert.setPositiveButton("Yes", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
                connection.resetDevice(writeListener);
            }
        });
        alert.setNegativeButton("No", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
            }
        });
        alert.show();
    }

    private void writeNewValue(String name, String value) {
        try {
            switch (name) {
                case "Overwrite Minor":
                    connection.overwriteMinor(Integer.valueOf(value), writeListener);
                    break;
                case "Enable Non Connectable Mode":
                    connection.enableNonConnectableMode(value, writeListener);
                    break;
                case "Overwrite Major":
                    connection.overwriteMajor(Integer.valueOf(value), writeListener);
                    break;
                case "Overwrite Proximity":
                    connection.overwriteProximity(UUID.fromString(value), writeListener);
                    break;
                case "Overwrite Password":
                    connection.overwritePassword(value, writeListener);
                    break;
                case "Overwrite Model Name":
                    connection.overwriteModelName(value, writeListener);
                    break;
                case "Overwrite Advertising Interval":
                    connection.overwriteAdvertisingInterval(Long.valueOf(value), writeListener);
                    break;
                case "Overwrite Power Level":
                    connection.overwritePowerLevel(Integer.valueOf(value), writeListener);
                    break;
                case "Restore Default Settings":
                    connection.restoreDefaultSettings(value, writeListener);
                    break;
            }
        } catch (IllegalArgumentException ex) {
            Toast.makeText(this, "Badly formatted input", Toast.LENGTH_LONG).show();
        }
    }

    private BeaconConnection.ConnectionListener listener = new BeaconConnection.ConnectionListener() {
        @Override
        public void onConnected() {
            if(DEBUG){
                Log.d(Constants.BEACONS_PLUGIN_TAG, "onConnected()");
            }
            mConnected = true;
            updateConnectionState(R.string.connected);
//            invalidateOptionsMenu();
        }

        @Override
        public void onAuthenticationSuccess(Beacon.Characteristics characteristics) {
            if(DEBUG){
                Log.d(Constants.BEACONS_PLUGIN_TAG, "onAuthenticationSuccess()");
            }
            mAuthenticated = true;
            mConnecting = false;
            KontaktDeviceControlActivity.this.characteristics = characteristics;
            updateAuthenticationState(R.string.authenticated);
            invalidateOptionsMenu();
        }

        @Override
        public void onAuthenticationFailure(int failureCode) {
            switch(failureCode){
                case BeaconConnection.FAILURE_UNKNOWN_BEACON:
                    Log.e(Constants.BEACONS_PLUGIN_TAG, "onAuthenticationFailure() + FAILURE_UNKNOWN_BEACON");
                    break;
                case BeaconConnection.FAILURE_WRONG_PASSWORD:
                    Log.e(Constants.BEACONS_PLUGIN_TAG, "onAuthenticationFailure() + FAILURE_WRONG_PASSWORD");
                    break;
            }
            mAuthenticated = false;
        }

        @Override
        public void onCharacteristicsUpdated(Beacon.Characteristics characteristics) {
            if(DEBUG){
                Log.d(Constants.BEACONS_PLUGIN_TAG, "onCharacteristicsUpdated()");
            }
            KontaktDeviceControlActivity.this.characteristics = characteristics;
        }

        @Override
        public void onErrorOccured(int errorCode) {
            switch(errorCode){
                case BeaconConnection.ERROR_SERVICES_DISCOVERY:
                    Log.e(Constants.BEACONS_PLUGIN_TAG, "onErrorOccured() + ERROR_SERVICES_DISCOVERY");
                    break;
                case BeaconConnection.ERROR_OVERWRITE_REQUEST:
                    Log.e(Constants.BEACONS_PLUGIN_TAG, "onErrorOccured() + ERROR_OVERWRITE_REQUEST");
                    break;
                case BeaconConnection.ERROR_AUTHENTICATION:
                    Log.e(Constants.BEACONS_PLUGIN_TAG, "onErrorOccured() + ERROR_AUTHENTICATION");
                    break;
                case BeaconConnection.ERROR_CHARACTERISTIC_READING:
                    Log.e(Constants.BEACONS_PLUGIN_TAG, "onErrorOccured() + ERROR_CHARACTERISTIC_READING");
                    break;
            }
        }

        @Override
        public void onDisconnected() {
            if(DEBUG){
                Log.d(Constants.BEACONS_PLUGIN_TAG, "onDisconnected()");
            }
            mConnected = false;
            mAuthenticated = false;
            updateConnectionState(R.string.disconnected);
            updateAuthenticationState(R.string.not_authenticated);
            invalidateOptionsMenu();
        }
    };

    private void updateConnectionState(final int resourceId) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                mConnectionState.setText(resourceId);
            }
        });
    }

    private void updateAuthenticationState(final int resourceId) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                mAuthenticationState.setText(resourceId);
            }
        });
    }

    private BeaconConnection.WriteListener writeListener = new BeaconConnection.WriteListener() {
        @Override
        public void onWriteSuccess() {
            if(DEBUG){
                Log.d(Constants.BEACONS_PLUGIN_TAG, "onWriteSuccess()");
            }
            Toast.makeText(KontaktDeviceControlActivity.this, "Write successful", Toast.LENGTH_SHORT).show();
        }

        @Override
        public void onWriteFailure() {
            if(DEBUG){
                Log.d(Constants.BEACONS_PLUGIN_TAG, "onWriteFailure()");
            }
            Toast.makeText(KontaktDeviceControlActivity.this, "Write unsuccessful", Toast.LENGTH_SHORT).show();
        }
    };
}
