package com.aware.plugin.beacons.view;

import android.content.SharedPreferences;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.v7.widget.Toolbar;
import android.view.View;
import android.widget.EditText;
import com.aware.plugin.beacons.R;
import com.aware.plugin.beacons.Singleton;
import com.aware.plugin.beacons.model.CalibrationReading;
import lecho.lib.hellocharts.model.Axis;
import lecho.lib.hellocharts.model.Line;
import lecho.lib.hellocharts.model.LineChartData;
import lecho.lib.hellocharts.model.PointValue;
import lecho.lib.hellocharts.view.LineChartView;

import java.util.ArrayList;
import java.util.List;

import static com.aware.plugin.beacons.Constants.LOG_CURVE_PARAMETER_A;
import static com.aware.plugin.beacons.Constants.LOG_CURVE_PARAMETER_B;
import static com.aware.plugin.beacons.Constants.LOG_CURVE_PARAMETER_N;

public class LogCurveHelloChartActivity extends AbstractHelloChartActivity {

    private EditText parameter_AEditText;
    private EditText parameter_bEditText;
    private EditText parameter_nEditText;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.log_curve_chart_layout);

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        getSupportActionBar().setTitle("Calibration chart");
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setBackgroundDrawable(new ColorDrawable(Color.rgb(8, 120, 165)));

        SharedPreferences sharedPref = PreferenceManager.getDefaultSharedPreferences(this);

        int b = Integer.valueOf(sharedPref.getString(LOG_CURVE_PARAMETER_B, "6"));
        double A = Double.valueOf(sharedPref.getString(LOG_CURVE_PARAMETER_A, "-65"));
        double n = Double.valueOf(sharedPref.getString(LOG_CURVE_PARAMETER_N, "1.5"));

        parameter_AEditText = (EditText) findViewById(R.id.parameter_AEditText);
        parameter_AEditText.setText(Double.toString(A));
        parameter_bEditText = (EditText) findViewById(R.id.parameter_bEditText);
        parameter_bEditText.setText(Integer.toString(b));
        parameter_nEditText = (EditText) findViewById(R.id.parameter_nEditText);
        parameter_nEditText.setText(Double.toString(n));

        findViewById(R.id.redrawBtn).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                generateData();
            }
        });

        chart = (LineChartView) findViewById(R.id.lineChartView);

        prepareChartData();

        generateData();

        chart.setViewportCalculationEnabled(false);

        resetViewport();
    }

    @Override
    protected void prepareChartData() {

        chartData = new ArrayList<>();

        List<PointValue> zeroMetersList = new ArrayList<>();
        for(CalibrationReading reading : Singleton.getInstance().getCalibrationData().get(0)){
            zeroMetersList.add(new PointValue(0, (float) reading.getRssi()));
        }
        chartData.add(zeroMetersList);

        List<PointValue> oneMeterList = new ArrayList<>();
        for(CalibrationReading reading : Singleton.getInstance().getCalibrationData().get(1)){
            oneMeterList.add(new PointValue(1, (float) reading.getRssi()));
        }
        chartData.add(oneMeterList);

        List<PointValue> twoMetersList = new ArrayList<>();
        for(CalibrationReading reading : Singleton.getInstance().getCalibrationData().get(2)){
            twoMetersList.add(new PointValue(2, (float) reading.getRssi()));
        }
        chartData.add(twoMetersList);
    }

    protected void generateData() {

        List<Line> lines = new ArrayList<>();
        int c = 0;
        for (List<PointValue> values : chartData) {

            Line line = new Line(values);
            line.setColor(colors[c++]);
            line.setShape(shape);
            line.setCubic(isCubic);
            line.setFilled(isFilled);
            line.setHasLabels(hasLabels);
            line.setHasLabelsOnlyForSelected(hasLabelForSelected);
            line.setHasLines(hasLines);
            line.setHasPoints(hasPoints);
            lines.add(line);
        }

        right = 3;

        double A = Double.parseDouble(parameter_AEditText.getText().toString());
        int b =  Integer.parseInt(parameter_bEditText.getText().toString());
        double n = Double.parseDouble(parameter_nEditText.getText().toString());

        List<PointValue> logCurveList = new ArrayList<>();
        for(float i = 0.1f; i < 3.0f; i += 0.1f){
            logCurveList.add(new PointValue(i, (float) ((-10 * n * logOfBase(b, i)) + A)));
        }
        lines.add(new Line(logCurveList).setColor(Color.DKGRAY).setCubic(false).setHasPoints(false));

        LineChartData data = new LineChartData(lines);

        if (hasAxes) {
            Axis axisX = new Axis();
            Axis axisY = new Axis().setHasLines(true);
            if (hasAxesNames) {
                axisX.setName("Distance");
                axisY.setName("RSSI");
            }
            data.setAxisXBottom(axisX);
            data.setAxisYLeft(axisY);
        } else {
            data.setAxisXBottom(null);
            data.setAxisYLeft(null);
        }

        data.setBaseValue(Float.NEGATIVE_INFINITY);

        chart.setLineChartData(data);
    }

    private double logOfBase(int base, double num) {
        return Math.log(num) / Math.log(base);
    }
}