package com.aware.plugin.beacons.view;

import android.app.Fragment;
import android.os.Bundle;
import android.util.Log;
import android.view.*;
import android.widget.LinearLayout;
import android.widget.Toast;
import com.aware.plugin.beacons.Constants;
import com.aware.plugin.beacons.R;
import com.aware.plugin.beacons.Singleton;
import com.aware.plugin.beacons.micromap.AreaVisualiser;
import com.aware.plugin.beacons.micromap.GMLParser;
import com.aware.plugin.beacons.model.AbstractBeacon;
import com.aware.plugin.beacons.semweb.SimilarBeaconsRunnable;
import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;
import java.io.IOException;

public class MapFragment extends Fragment {

    private static final int MAP_MY_ROOM = 0;
    private static final int MAP_LAB_316 = 1;

    private AreaVisualiser lab316AreaVisualiser;
    private AreaVisualiser myRoomAreaVisualiser;
    private int currentMap = MAP_MY_ROOM;

    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        setHasOptionsMenu(true);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {

        final View rootView = inflater.inflate(R.layout.map_fragment_layout, container, false);

        try {
            lab316AreaVisualiser = new AreaVisualiser(getActivity(),
                    GMLParser.parseMicroMap("maps/lab316/conf.mmc", getActivity()).getAreas().getFirst(),
                    null);
            myRoomAreaVisualiser = new AreaVisualiser(getActivity(),
                    GMLParser.parseMicroMap("maps/my_room/conf.mmc", getActivity()).getAreas().getFirst(),
                    null);

        } catch (ParserConfigurationException | SAXException | IOException e) {
            Log.e(Constants.BEACONS_PLUGIN_TAG, e.getMessage());
        }

        ((LinearLayout) rootView.findViewById(R.id.map_layout)).addView(myRoomAreaVisualiser);

        return rootView;
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater){
        inflater.inflate(R.menu.map_fragment_menu, menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.map_menu_my_room:
                currentMap = MAP_MY_ROOM;
                redrawMap();
                return true;
            case R.id.map_menu_lab316:
                currentMap = MAP_LAB_316;
                redrawMap();
                return true;
            case R.id.map_menu_find_closest:
                redrawMap();
                break;
            case R.id.map_menu_find_similar:
                AbstractBeacon closestBeacon = Singleton.getInstance().getClosestBeacon();
                if(closestBeacon != null){
                    new Thread(new SimilarBeaconsRunnable(this)).start();
                }else{
                    Toast.makeText(getActivity(), "Closest beacon not found yet", Toast.LENGTH_SHORT).show();
                }
                break;
        }
        return super.onOptionsItemSelected(item);
    }

    public void redrawMap(){
        View view = getView();
        if(view != null) {
            LinearLayout layout = (LinearLayout) getView().findViewById(R.id.map_layout);
            layout.removeAllViews();
            switch (currentMap) {
                case MAP_MY_ROOM:
                    layout.addView(myRoomAreaVisualiser);
                    break;
                case MAP_LAB_316:
                    layout.addView(lab316AreaVisualiser);
                    break;
            }
        }
    }
}