package com.aware.plugin.beacons.view;

import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.support.v7.widget.Toolbar;
import com.aware.plugin.beacons.R;
import com.aware.plugin.beacons.Singleton;
import com.aware.plugin.beacons.model.CalibrationReading;
import lecho.lib.hellocharts.model.Axis;
import lecho.lib.hellocharts.model.Line;
import lecho.lib.hellocharts.model.LineChartData;
import lecho.lib.hellocharts.model.PointValue;
import lecho.lib.hellocharts.view.LineChartView;

import java.util.ArrayList;
import java.util.List;

public class RegularHelloChartActivity extends AbstractHelloChartActivity {

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.regular_chart_layout);

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        getSupportActionBar().setTitle("Calibration chart");
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setBackgroundDrawable(new ColorDrawable(Color.rgb(8, 120, 165)));

        chart = (LineChartView) findViewById(R.id.lineChartView);

        prepareChartData();

        generateData();

        chart.setViewportCalculationEnabled(false);

        resetViewport();
    }

    @Override
    protected void prepareChartData() {

        chartData = new ArrayList<>();

        int x = 0;

        List<PointValue> zeroMetersList = new ArrayList<>();
        for(CalibrationReading reading : Singleton.getInstance().getCalibrationData().get(0)){
            zeroMetersList.add(new PointValue(x++, (float) reading.getRssi()));
        }
        chartData.add(zeroMetersList);

        List<PointValue> oneMeterList = new ArrayList<>();
        for(CalibrationReading reading : Singleton.getInstance().getCalibrationData().get(1)){
            oneMeterList.add(new PointValue(x++, (float) reading.getRssi()));
        }
        chartData.add(oneMeterList);

        List<PointValue> twoMetersList = new ArrayList<>();
        for(CalibrationReading reading : Singleton.getInstance().getCalibrationData().get(2)){
            twoMetersList.add(new PointValue(x++, (float) reading.getRssi()));
        }
        chartData.add(twoMetersList);
    }

    protected void generateData() {

        List<Line> lines = new ArrayList<>();
        int i = 0;
        for (List<PointValue> values : chartData) {

            Line line = new Line(values);
            line.setColor(colors[i++]);
            line.setShape(shape);
            line.setCubic(isCubic);
            line.setFilled(isFilled);
            line.setHasLabels(hasLabels);
            line.setHasLabelsOnlyForSelected(hasLabelForSelected);
            line.setHasLines(hasLines);
            line.setHasPoints(hasPoints);
            lines.add(line);
        }

        right = 0;
        for(List<PointValue> list : chartData){
            right += list.size();
        }

        LineChartData data = new LineChartData(lines);

        if (hasAxes) {
            Axis axisX = new Axis();
            Axis axisY = new Axis().setHasLines(true);
            if (hasAxesNames) {
                axisX.setName("Index");
                axisY.setName("RSSI");
            }
            data.setAxisXBottom(axisX);
            data.setAxisYLeft(axisY);
        } else {
            data.setAxisXBottom(null);
            data.setAxisYLeft(null);
        }

        data.setBaseValue(Float.NEGATIVE_INFINITY);

        chart.setLineChartData(data);
    }
}